<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/

/*
 * Created on 25.04.2005
 * 
 * This script adds represents the web formular for adding items.
 * If the parameter itemID is set, then this means changing the item
 * instead of creating a new one.
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 0.9
 */

 require_once('./classes/DisplayView.class.php');
 require_once($config["proj_path"].'/classes/ItemParser.class.php');
 require_once($config["proj_path"].'/classes/FolderManager.class.php');
 require_once($config["proj_path"].'/classes/OptionManager.class.php');
 require_once($config["proj_path"].'/classes/ThumbCreator.class.php');

 define("ITEM_TITLE_SESSION_NAME","item_title");
 define("ITEM_LOCATION_SESSION_NAME","item_location");
 define("ITEM_CATEGORY_SESSION_NAME","item_category");
 define("ITEM_TEXT_SESSION_NAME","item_text");
 
 class AddItemView extends DisplayView {
 	var $item_parser;
 	var $folder_manager;
 	var $option_manager;
 	var $thumb_creator;
 	
 	var $edit_item;
 	var $item_data;
 	
 	function AddItemView() {
 		global $config;
 		parent::DisplayView();
 		$this->edit_item = false;
 		$this->item_data = false;
 		
 		// check if the item should be changed instead of being created
 		if(isset($_REQUEST["itemID"]) && $_REQUEST["itemID"] != "") {
 		    $this->edit_item = true;
 		    $this->setTitle($this->lang_main["change_item_title"]);
 		    $this->item_data = $this->item_manager->getItemById($_REQUEST["itemID"]);
 		    if(!$this->item_data) {
	 			$this->showErrorPage($this->lang_error["item_not_existing"]);
	 			exit;
	 		}
 		} else
 		    $this->setTitle($this->lang_main["create_item_title"]);
 		
 		$this->item_parser = new ItemParser($this->dbCon, $config, $this->options);
 		$this->option_manager = new OptionManager($this->dbCon, $config,$this->options);
 		$this->folder_manager = new FolderManager();
 		$this->thumb_creator = new ThumbCreator($this->dbCon,$config,$this->options);
 	}
 	
 	function display() {
 		if(!$this->checkRights()) exit;
 		
 		$this->handlePosts();
 		$this->showForm();
 		
 		$this->my_smarty->display($this->options["template"].'/item_form.tpl');
 	}
 	
 	function showForm() {
 		global $config;
 		$user_path = $config["proj_path"]."media/temp/".$this->user["userID"]."/";
 		
 		if(isset($_REQUEST["itemID"]) && $_REQUEST["itemID"] != "") {
 		     $this->my_smarty->assign('create_item_title',$this->lang_main["change_item_title"]);
 		     $this->my_smarty->assign('do_command',"edit_item");
 		     $this->my_smarty->assign('itemID',$_REQUEST["itemID"]);
		     $this->my_smarty->assign('attach_url',"./media/cat".$this->item_data["catID"]."/item".$this->item_data["itemID"]."/");
 		     
 		     $item_path = $config["proj_path"]."media/cat".$this->item_data["catID"]."/item".$this->item_data["itemID"]."/";
 		     $files = $this->folder_manager->listFilesByFolder($item_path);
		     $this->my_smarty->assign('attachements',$files);
 		     
 		     if((!isset($_POST["do"]) || $_POST["do"] != "edit_item")
 		        && (!isset($_POST["itemID"]) || $_POST["itemID"] == "")) {
 		     	$this->loadItemToForm($this->item_data);
 		     }
 		} else {
 		   $this->my_smarty->assign('create_item_title',$this->lang_main["create_item_title"]);
 		   $this->my_smarty->assign('do_command',"add_item");
		   $files = $this->folder_manager->listFilesByFolder($user_path);
		   $this->my_smarty->assign('attachements',$files);
		   $this->my_smarty->assign('attach_url',"./media/temp/".$this->user["userID"]);
 		}
 		
 		$this->my_smarty->assign('template_path',"templates/".$this->options["template"]);
 		$this->my_smarty->assign('lang_js',$this->lang_main["lang_js"]);
		$this->my_smarty->assign('title_label',$this->lang_main["item_title_label"]);
		$this->my_smarty->assign('location_label',$this->lang_main["item_location_label"]);
		$this->my_smarty->assign('category_label',$this->lang_main["item_category_label"]);
		$this->my_smarty->assign('create_item_button',$this->lang_main["create_item_button"]);
		$this->my_smarty->assign('files_label',$this->lang_main["files_label"]);
		$this->my_smarty->assign('userfile_label',$this->lang_main["choose_file_label"]);
		$this->my_smarty->assign('del_button',$this->lang_main["files_delete_button"]);
		$this->my_smarty->assign('add_file_button',$this->lang_main["add_file_button"]);
		$this->my_smarty->assign('userfile_max',$this->lang_main["max_file_size"]." ".$this->options["max_media_size"]." KB");
		$this->my_smarty->assign('item_tooltips',$this->lang_main["item_tooltips"]);
		
		if($this->options["max_files"] != 0)
			$this->my_smarty->assign('add_files_allowed',"true");
		
		$this->my_smarty->assign('categories',$this->cat_manager->getAllCategoriesWithCount($this->item_manager));
		
	 	foreach($_POST as $key=>$value) {
	 		if($key=="item_text_js" || $key=="rte1")
		       $this->my_smarty->assign('text_content',$this->rteSafe($value));
		    $this->my_smarty->assign($key.'_value',$value);
	 	}
 	}
 	
 	/**
 	 * Loads the data from the database to the form.
 	 * 
 	 * @param itemID The ID of the item.
 	 */
 	function loadItemToForm($item_data) {
 		global $config;
 		
 		// now fill the form
		$this->my_smarty->assign('item_title_value',$item_data["title"]);
		$this->my_smarty->assign('item_location_value',$item_data["location"]);
		$this->my_smarty->assign('item_category_value',$item_data["catID"]);
		
		$this->my_smarty->assign('text_content',$this->rteSafe($item_data["item_text"]));
		$this->my_smarty->assign('item_text_value',$item_data["item_text"]);
		
		if($item_data["main_img"] != null) {
		    $main_img = $this->item_manager->getMediaByID($item_data["main_img"]);
		    $this->my_smarty->assign('main_img_value',$main_img["filename"]);
		}
		
 	}
 	
 	function rteSafe($strText) {
		//returns safe code for preloading in the RTE
		$tmpString = $strText;
		
		//convert all types of single quotes
		$tmpString = str_replace(chr(145), chr(39), $tmpString);
		$tmpString = str_replace(chr(146), chr(39), $tmpString);
		$tmpString = str_replace("'", "&#39;", $tmpString);
		
		//convert all types of double quotes
		$tmpString = str_replace(chr(147), chr(34), $tmpString);
		$tmpString = str_replace(chr(148), chr(34), $tmpString);
		
		//replace carriage returns & line feeds
		$tmpString = str_replace(chr(10), " ", $tmpString);
		$tmpString = str_replace(chr(13), " ", $tmpString);
		
		return $tmpString;
	}
 	
 	function handlePosts() {
 		global $config;
 		
 		if(isset($_POST["file_button"])) {
 		    $path = $config["proj_path"]."media/temp/".$this->user["userID"]."/";
 		    if($this->edit_item)
 		       $path = $config["proj_path"]."media/cat".$this->item_data["catID"]."/item".$this->item_data["itemID"]."/";
 		    
 			$this->saveFileToTemp($path);
 			return;
 		}
 		if(isset($_POST["del_button"])) {
 			$this->deleteTempFiles();
 			return;
 		}
 		
 		foreach($_POST as $key => $value) {
 			
 			$rest = strstr($key,"addsp_");
 			if($rest != false && strstr($key,"2") == false) {
 				
 				$isImg = strstr($rest,"img_");
 				if(!$isImg) {
 					$rest2 = $isImg = strstr($rest,"else_");
 					$_POST["item_text"] = $_POST["item_text"]." [a:".$_POST["2".$key].":".$_POST["2".$key]."]";
 				} else {
 					$_POST["item_text"] = $_POST["item_text"]." [i:".$_POST["2".$key]."]";
 				}
 				return;
 			}
 		}
 		if(!isset($_POST["do"])) return;
 		if($_POST["do"]=="add_item" || $_POST["do"]=="edit_item") {
 			if($this->checkInputfields())
 			   $this->createItem();
 		}
 	}
 	
 	/**
 	 * Deletes the selected temp files.
 	 */
 	function deleteTempFiles() {
 		global $config;
 		
 		$user_path = $config["proj_path"]."media/temp/".$this->user["userID"]."/";
 		if($this->edit_item)
 		       $user_path = $config["proj_path"]."media/cat".$this->item_data["catID"]."/item".$this->item_data["itemID"]."/";
 		    
 		if(!isset($_POST["del_files"])) return;
 		foreach($_POST["del_files"] as $file_name) {
 			if($this->edit_item) {
 				if($file_name == $_POST["main_img"])
 				    $this->item_manager->setMainImg($this->item_data["itemID"],"NULL");
 			    $this->item_manager->deleteMedia($file_name,$this->item_data);
 			} else
 			    unlink($user_path . $file_name);
 		}
 	}
 	
 	/**
 	 * Saves a file to the users temp folder.
 	 */
 	function saveFileToTemp($path) {
 		global $config;
 		
 		if($this->options["max_files"] >= 0) {
 			$user_path = $config["proj_path"]."media/temp/".$this->user["userID"]."/";
	 		if($this->edit_item)
	 		       $user_path = $config["proj_path"]."media/cat".$this->item_data["catID"]."/item".$this->item_data["itemID"]."/";
 			$media_count = count($this->folder_manager->listFilesByFolder($user_path));
 			
 			if($media_count+1 > $this->options["max_files"]) {
 				$this->showError($this->lang_error["max_files_reached"]." ".$this->options["max_files"]);
 				return;
 			}
 		}
 		
 		if($_FILES["userfile"]["error"] != 0) return;
 		if(($_FILES["userfile"]["size"]/1024) > $this->options["max_media_size"]){
 			$this->showError($this->lang_error["file_size_to_big"]." (".($this->options["max_media_size"])." KB)");
			return;
 		}
    	
 		if(!file_exists($path)){
    		mkdir($path);
    	}
    	
    	$file_name = $_FILES["userfile"]["name"];
    	$pos = strpos($file_name, ".");
    	$extension = "";
    	$file = $file_name;
    	if($pos != false) {
	    	$file = substr($file_name, 0, $pos);
			$extension = strtolower(substr($file_name, $pos+1));
			if(!$this->option_manager->formatAllowed($extension)) {
				$this->showError($this->lang_error["file_format_not_allowed"]);
				return;
			}
    	}
    	
    	// rename file as long as no file with the same name exists
    	$file_path = $path . $file .".". $extension;
    	$count = 0;
    	$new_name = $file_name;
    	while(file_exists($file_path)) {
    		$count++;
    		$file_path = $path . $file . $count .".". $extension;
    		$new_name = $file . $count .".". $extension;
    	}
    	move_uploaded_file($_FILES["userfile"]["tmp_name"],$file_path);
    	
    	if($this->edit_item) {
    		$this->item_manager->addMediaToDB($this->item_data["itemID"],$this->item_data["catID"],$new_name,false);
    	}
 	}
 	
 	/**
 	 * Creates the item_data array and sends it to the item parser.
 	 */
 	function createItem() {
 		global $config;
 		$item_data = array();
	 	$item_data["user"] = $this->user["username"];
	 	$item_data["title"] = $_POST["item_title"];
 		$item_data["item_editor"] = WEBSITE_EDITOR;
 		
	 	if(isset($_POST["rte1"])) {
	 		
	 		$item_data["item_text"] = $this->rteSafe($_POST["rte1"]);//$this->textToSpecialSyntax($this->rteSafe($_POST["rte1"]));
	 		$item_data["item_editor"] = RTE_EDITOR;
	 	}
	 	else if(isset($_POST["item_text_js"])) {
	 		$item_data["item_text"] = $_POST["item_text_js"];//$this->textToSpecialSyntax($this->rteSafe($_POST["item_text_js"]));
	 		$item_data["item_editor"] = WEBSITE_EDITOR;
	 	}
	 	else if(isset($_POST["item_text"])) $item_data["item_text"] = $_POST["item_text"];
	 	
	 	if (isset($_POST["item_location"]) && $_POST["item_location"]!="") $item_data["location"] = $_POST["item_location"];
	 	if (isset($_POST["item_category"]) && $_POST["item_category"]!="") $item_data["category"] = $_POST["item_category"];
	 	
	 	if($this->edit_item) {
	 		$item_data["itemID"] = $this->item_data["itemID"];
	 		$itemID = $this->item_parser->parseItem($item_data);
	 		$user_path = $config["proj_path"]."media/cat".$this->item_data["catID"]."/item".$this->item_data["itemID"]."/";
	 	} else {
	 		$itemID = $this->item_parser->parseItem($item_data,0);
	 		$user_path = $config["proj_path"]."media/temp/".$this->user["userID"]."/";
	 	}
	 	$files = $this->folder_manager->listFilesByFolder($user_path);
	 	foreach($files as $file) {
	 		$file_path = $user_path.$file["name"];
			
	 		$media_data[0]["user_tmp_name"] = $file_path;
	 		$media_data[0]["name"] = $file["name"];
	 		if(isset($_POST["main_img"]) && $file["name"] == $_POST["main_img"])
	 			$media_data[0]["main_img"] = "true";
	 	    else
	 		   $media_data[0]["main_img"] = "false";
	 		   
	 		if($this->edit_item) {
	 			$is_main = false;
	 			
	 			if(isset($media_data[0]["main_img"]) && $media_data[0]["main_img"] == "true")
	 			   $is_main = true;
	 			$this->item_manager->addMediaToDB($this->item_data["itemID"],$this->item_data["catID"],$file["name"],$is_main);
	 			if(!file_exists($user_path."thumb/tn_".$file["name"]))
	 			   $this->thumb_creator->createThumbnail($user_path, $file["name"]);
	 		} else {
		 		$this->item_parser->addMedia($itemID,$media_data);
		 		unlink($file_path);
	 		}
	 	}
	 	
	 	if($this->edit_item) {
	 	    $this->createHeader();
	 		$this->showSuccessPage($this->lang_main["edit_item_success"]);
	 	   exit;	
	 	}
	 	
	 	// make necessary changes to the images
	 	$item = $this->item_manager->getItemByID($itemID,true);
	 	$old_path = "/\/media\/temp\/".$this->user["userID"]."\//";
	 	$real_folder = "/media/cat".$item["catID"]."/";
	 	$real_folder = $real_folder."item".$itemID."/"; 
	 	$new_text = $this->replaceImgFolder($item["item_text"],$old_path,$real_folder);
	 	$this->item_manager->updateItemText($itemID,$new_text);
	 	// at last set the item visible
	    $this->item_manager->setVisible($itemID,true);
	 	$this->createHeader();
	 	$this->showSuccessPage($this->lang_main["create_item_success"]);
    	exit;
 	}
 	
 	/**
 	 * Replaces all the temp img folders in the $text with the real folders.
 	 */
 	function replaceImgFolder($text,$temp_folder,$real_folder) {
 		$new_text = preg_replace($temp_folder,$real_folder,$text);
 		return $new_text;
 	}
 	
 	/**
 	 * Modifies the text to fit to the SpecialSyntax.
 	 */
// 	private function textToSpecialSyntax($text) {
// 		$new_text = preg_replace("/<img[^>]+src=(\S){0,1}(\"){0,1}([^\"]*?)(\"){1,1}[^>]+replace_img[^>]+\/>/e","'[i:'.substr('\\3', strrpos('\\3','/')+1,strlen(substr('\\3', strrpos('\\3','/')+1))-1).']'",$text);
// 		$new_text = preg_replace("/<img[^>]+replace_img[^>]+src=(\S){0,1}(\"){0,1}([^\"]*?)(\"){1,1}[^>]+\/>/e","'[i:'.substr('\\3', strrpos('\\3','/')+1,strlen(substr('\\3', strrpos('\\3','/')+1))-1).']'",$new_text);
// 		
// 		return $new_text;
// 	}
 	
 	/**
 	 * Checks if the user has the right to view the page.
 	 * If not it returns false.
 	 */
 	function checkRights() {
 		if($this->valid_user && $this->edit_item) {
 			if(!$this->rank_manager->editItemAllowed($this->user,$this->item_data)) {
 				$this->showErrorPage($this->lang_error["edit_item_not_allowed"]);
 			    return false;
 			}
 		} else if($this->valid_user && !$this->rank_manager->addItemAllowed($this->user)) { // user without rights
 			$this->showErrorPage($this->lang_error["add_item_forbidden"]);
 			return false;
 		} else if(!$this->valid_user ) { // user not logged in
 			$this->showErrorPage($this->lang_error["not_logged_in"]);
 			return false;
 		} else if (!$this->cat_manager->getNumberOfCategories() > 0){ //if no category exists yet
 			$this->showErrorPage($this->lang_error["no_existing_category"]);
 			return false;
 		}
 		return true;
 	}
 	
 	function checkInputfields() {
	 	$missing_field = false;
	 	
	 	if(!isset($_POST["item_title"]) || $_POST["item_title"] == "") {
	 		$missing_field = true;
	 		$this->my_smarty->assign("item_title_class","red_border");
	 	}
	 	
	 	if(isset($_POST["rte1"]) && $_POST["rte1"] == "") {
	 		$missing_field = true;
	 		$this->my_smarty->assign("rte_border","true");
	 	}
	 	
	 	if(isset($_POST["item_text"]) && $_POST["item_text"] == "") {
	 		$missing_field = true;
	 		$this->my_smarty->assign("item_text_class","red_border");
	 	}
	 	
	 	if(isset($_POST["item_text_js"]) && $_POST["item_text_js"] == "") {
	 		$missing_field = true;
	 		$this->my_smarty->assign("rte_border","true");
	 	}
	 	
	 	if($missing_field) {
	 		$this->showError($this->lang_error["missing_input_fields"]);
	 		return false;
	 	}
	 	return true;
	}
	
	function getScriptName() {
		global $php_extension;
   	    return "additem.".$php_extension;
    }
 }
 
 $add_item_view = new AddItemView();
 $add_item_view->display();
 
?>
