<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/

/**
 * Created on 04.05.2005
 *
 * This script shows a specific category or an overview over all categories.
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 0.1
 */

 require_once('./classes/DisplayView.class.php');
 require_once('./classes/SpecialSyntaxParser.class.php');
 require_once('./classes/CommentManager.class.php');
 require_once('./classes/ThumbCreator.class.php');
 
 
class CatView extends DisplayView {
	
 	var $cat_data;
 	var $truncate_size = 800;
 	var $thumb_creator;
 	
 	function CatView() {
 		global $config;
 		parent::DisplayView();
 		$this->thumb_creator = new ThumbCreator($this->dbCon,$config,$this->options);
 	}
 	
 	function display() {
 		global $config;
 		if($this->dbCon->itsTimeToCheck() && $this->options["email_check_on"] == 1) {
			require_once($config["proj_path"].'/classes/CheckMail.class.php');
			$checkmail = new CheckMail($this->dbCon, $config);
			$checkmail->checkForMails();
 		}
 		if(isset($_REQUEST["do"])) $this->handleDoCommands();
 		if(!$this->isValidID()) return;
 		
 		// delete unused items
 		$this->item_manager->deleteUnusedItems();
 		
 		$this->showCategory();
 		
 		$this->my_smarty->display($this->options["template"].'/category.tpl');
 	}
 	
 	function handleDoCommands(){
 		if($_REQUEST["do"] = "delete" && isset($_REQUEST["deleteID"])) $this->showConfirmDialogue();
 		else if($_REQUEST["do"] = "delete_confirmed" && isset($_REQUEST["confirm_id"])) $this->deleteCat();
 	}
 	
 	function isValidID() {
 		global $config;
 		
 		if(!isset($_REQUEST["catID"])) {
 			// if no id is specified show all categories
		 	$this->my_smarty->assign('category_overview',$this->lang_main["category_overview"]);
		 	$categories = $this->cat_manager->getAllCategoriesWithCount($this->item_manager);
		 	if(!$categories) {
		 		$this->my_smarty->assign('no_existing_category', $this->lang_error["no_existing_category_available"]);
		 	}
		 	else $this->my_smarty->assign('categories', $categories);
		 	
		 	//check if the user has the rights to delete a category
		 	if($this->checkDeleteRights()) $this->my_smarty->assign("right_to_delete", "true");
		 	
		    $this->my_smarty->display($this->options["template"].'/category_list.tpl');
		    return false;
 		}
 		else {
 			// if "id" is specified check if a category with this id exists
 			$this->cat_data = $this->cat_manager->getCatByID($_REQUEST["catID"]);
 			if(!$this->cat_data){
 				// if the category with this id don't exists in the database show an error
			 	$this->my_smarty->assign('error_title',$this->lang_error["category_not_existing_title"]); 
			    $this->my_smarty->assign('error_message',$this->lang_error["category_not_existing"]);
			    $this->my_smarty->display($this->options["template"].'/error.tpl');
			    return false;
			}
			return true;
 		}
 	}
 	
 	function showCategory(){
 		global $config;
 		global $php_extension;
 		$special_syntax_parser = new SpecialSyntaxParser($this->dbCon,$config);
 		
 		$this->my_smarty->assign('cat_name',$this->cat_data["catname"]);
		$this->setTitle($this->cat_data["catname"]);
		
		$startID = 0;
		if(isset($_REQUEST["start"])) $startID = $_REQUEST["start"];
		$item_count = $this->item_manager->getItemCount($_REQUEST["catID"]);
		if($item_count == 0){
			$this->my_smarty->assign('category_empty_message', $this->lang_main["category_empty_message"]);
			$this->my_smarty->display($this->options["template"].'/category.tpl');
		  	exit;
	  	}
		$comment_manager = new CommentManager($this->dbCon,$config,$this->options);
		$items = $this->item_manager->getAllItemsByCategory($_REQUEST["catID"],$startID,$this->options["max_items_per_page"]);
		$items = $this->item_manager->createDisplayItems($items,$special_syntax_parser,$comment_manager,$this->thumb_creator, $this->lang_main);
		if($this->options["comments_allowed"] == 1) // are comments allowed?
		    	$this->my_smarty->assign('comments_allowed',"true");
		
		$this->my_smarty->assign('items',$items);
		$this->my_smarty->assign("truncate_size",$this->truncate_size);
		$this->my_smarty->assign('comments_label',$this->lang_main["comments_label"]);
		$this->my_smarty->assign('catID', $_REQUEST["catID"]);
		
		
		
		// show rss link
		$rss_path = "feed.".$php_extension."?id=".$this->cat_data["catID"];
		$this->my_smarty->assign('rss_url', $rss_path);
		$this->my_smarty->assign('rss_title', $this->cat_data["catname"]);
		
		// next button to navigate to the next items of this category
		if($item_count > $startID + $this->options["max_items_per_page"]) {
			$this->my_smarty->assign('item_next_value', $startID + $this->options["max_items_per_page"]);
			$this->my_smarty->assign('item_next_button', $this->lang_main["item_next_button"]);
		}
		
		$pages = ceil($item_count / $this->options["max_items_per_page"]);
		$pages_array = range(1,$pages);
		if($startID <= 0){
			$act_page = 1;
		}
		else $act_page = (int)($startID / $this->options["max_items_per_page"]) + 1;
		if($item_count > $this->options["max_items_per_page"]){
			$this->my_smarty->assign("show_first_page_navi", "true");
		}
		$this->my_smarty->assign("pages", $pages_array);
		$this->my_smarty->assign("act_page", $act_page);
		$this->my_smarty->assign("step", $this->options["max_items_per_page"]);
		
		// back button to navigate back
		$backID = $startID - $this->options["max_items_per_page"]; 
		if($backID < 0) $backID = 0;
		if($startID != 0){
			$this->my_smarty->assign('item_back_value', $backID);
			$this->my_smarty->assign('item_back_button', $this->lang_main["item_back_button"]);
		}
 	}
 	
 	/**
 	 * Checks if the user has the right to delete a category.
 	 * If not it returns false.
 	 */
 	function checkDeleteRights() {
 		if($this->valid_user && $this->rank_manager->deleteCategoryAllowed($this->user)) { // user with rights
 			return true;
 		} 
 		return false;
 	}
 	
 	function showConfirmDialogue() {
		if(!$this->checkDeleteRights()) return;
		$this->my_smarty->assign('yes_button',$this->lang_main["yes_button"]);
		$this->my_smarty->assign('no_button',$this->lang_main["no_button"]);

	    $this->my_smarty->assign('dialogue_url',$this->getScriptName());
	    $this->my_smarty->assign('dialogue_do_value',"delete_confirmed");
		$this->my_smarty->assign('confirm_id',$_REQUEST["deleteID"]);
 		$this->my_smarty->assign('dialogue_title',$this->lang_main["dialogue_delete_title"]);
		$this->my_smarty->assign('dialogue_text',$this->lang_main["dialogue_delete_category"]);
 	}
 	
 	function deleteCat(){
 		if(!$this->checkDeleteRights()) return;
 		if(isset($_POST["yes_button"])){
 			if(!$this->cat_manager->isExistingCategory($_POST["confirm_id"])) return;
 			if($this->cat_manager->deleteCategory($_POST["confirm_id"], $this->item_manager)){
 				$this->my_smarty->assign("success_message", $this->lang_main["dialogue_delete_category_success"]);
 				$this->my_smarty->assign("success_title", $this->lang_main["dialogue_delete_title"]);
 				$this->createHeader();
 			}
 		}
 	}
 	
 	
 	
 	function getScriptName() {
 		global $php_extension;
   	    return "cat.".$php_extension;
    }
}

 $cat_view = new CatView();
 $cat_view->display();
?>