<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/


/**
 * Created on 02.06.2005
 * 
 * This class provides some functions to handle with categories
 * 
 * To use this class, it is required, that a connection
 * to the I-Reporter Database has already been established.
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 0.1
 */

class CategoryManager {
	
    var $dbCon;
    var $config;
 
    function CategoryManager($dbCon, $config) {
    	$this->dbCon = $dbCon;
    	$this->config = $config;
    }
    
    /**
     * Returns an array containing the attributes of a category
     * or false if no category with the ID exists.
     * 
     * @param $category_id The ID of the category.
     * @return An array like this: array["catID"], array["catname"], array["rss_items_count"].
     */
    function getCatByID($category_id) {
    	$result = $this->dbCon->selectFromDB("select * from ".$this->config["db_prefix"]."category where catID = ".$category_id);
    	while ($row = mysql_fetch_array($result)){
    		return $row;
    	}
    	return false;
    }
    
    /**
     * This function gets the default category out of the database
     * @return the default category
     */
     function getDefaultCatID(){
     	$result = $this->dbCon->selectFromDB("Select default_cat from ".$this->config["db_prefix"]."options");
    	$array = mysql_fetch_array($result);
    	$defaultCatID = $array["default_cat"];
    	
    	$num_rows = $this->dbCon->selectNumRows("Select * from ".$this->config["db_prefix"]."category where catID=$defaultCatID");
    	if ($num_rows > 0) return $defaultCatID;
    	
    	$result = $this->dbCon->selectFromDB("Select * from ".$this->config["db_prefix"]."category");
    	if ($array = mysql_fetch_array($result)){
    		$catID = $array["catID"];
    		return $catID;
    	}
    	return false;
     }
     
     /**
      * This function gets the total number of categories that are in the database
      * @return Returns the number of categories
      */
     function getNumberOfCategories(){
     	$result = $this->dbCon->selectFromDB("Select count(*) as cat_count from ".$this->config["db_prefix"]."category");
     	while($array = mysql_fetch_array($result)){
     		$cat_count = $array["cat_count"];
     		return $cat_count;
     	}
     	return 0;
     }
     
     /**
      * Returns an array containing all categories ordered by name including the number
      * of their items.
      * <br><br>
      * <b>example</b>:
      * <br>
      * $test = getAllCategoriesWithCount();
      * $test[0]["item_count"]; <- the number of items in category 0
      */
     function getAllCategoriesWithCount($item_manager) {
     	$result = $this->dbCon->selectFromDB("Select * from ".$this->config["db_prefix"]."category order by catname");
     	$categories = array();
     	while($row = mysql_fetch_array($result)) {
     		if($item_manager != false) $row["item_count"] = $item_manager->getItemCount($row["catID"]);
     		$categories[count($categories)] = $row;
     	}
     	usort($categories, array ("CategoryManager", "cmp"));
     	if(count($categories) > 0) return $categories;
     	return false;
     }
     
     function cmp($a, $b) {
	    return strcmp($a["catname"], $b["catname"]);
	 }
     
     /**
      * Returns an array containing all categories ordered by the number
      * of their items. The first category is the one with the most items
      * 
      */
     function getAllCategoriesOrderedByCount() {
     	$sql = "SELECT c.catID, count(*) as item_count, max(catname) as catname FROM ".$this->config["db_prefix"]."category c NATURAL JOIN ".$this->config["db_prefix"]."item where item_visible = 1 GROUP BY c.catID HAVING count(*) > 0 order by item_count desc limit 0, 3";
     	$result = $this->dbCon->selectFromDB($sql);
     	$categories = array();
     	while($row = mysql_fetch_array($result)) {
     		$categories[count($categories)] = $row;
     	}
     	if(count($categories) > 0) {
//     		usort($categories, array("CategoryManager", "cmp"));
     		return $categories;
     	}
     	return false;
     }
     
//     public static function comp($a, $b){
//     	if($a["item_count"] == $b["item_count"]) return 0;
//     	return ($a["item_count"] > $b["item_count"]) ? -1 : 1;
//     }
     
     /**
     * Checks if the given category exists in the database.
     * If yes it returns the catID of this category.
     * If no it returns the ID of the default category specified in the options
     * 
     * @param $cat either the id or the name of the category
     */
    function isExistingCategory($cat){
    	
    	if (preg_match("/[0-9]+/", $cat)){
    		$result = $this->dbCon->selectFromDB("Select * from ".$this->config["db_prefix"]."category where catID = ".$cat);
	    	while ($array = mysql_fetch_array($result)){
	    		 return $array["catID"];
	    	}
	    	return $this->getDefaultCatID();
    	}
    	else {
    		$result = $this->dbCon->selectFromDB("Select * from ".$this->config["db_prefix"]."category where catname like '".$cat."'");
	    	while ($array = mysql_fetch_array($result)){
	    		 return $array["catID"];
	    	}
	    	return $this->getDefaultCatID();
    	}
    }
    
    /**
     * Checks if the given category name exists in the database.
     * If yes it returns true. Else false
     * 
     * @param $catname the name of the category
     */
    function isExistingCategoryName($catname, $catID = ""){
    	if($catID != "") $result = $this->dbCon->selectFromDB("Select * from ".$this->config["db_prefix"]."category where catname like '".$catname."' and catID != $catID");
    	else $result = $this->dbCon->selectFromDB("Select * from ".$this->config["db_prefix"]."category where catname = '".$catname."'");
    	while ($array = mysql_fetch_array($result)){
    		 return true;
    	}
    	return false;
    }
    
    function deleteCategory($catID, $item_manager){
    	$items = $item_manager->getAllItemsByCategory($catID, 0, "all");
    	if($items){
    		foreach($items as $item){
	    		if(!$item_manager->deleteItem($item["itemID"])) return false;
	    	}
    	}
    	
    	$sql = "DELETE FROM ".$this->config["db_prefix"]."category WHERE catID = ".$catID;
    	if(!$this->dbCon->queryDB($sql)) return false;
    	
    	$path = $this->config["proj_path"]."media/cat".$catID;
    	if(file_exists($path)){
    		if(!@rmdir($path)) return false;
    	}
    	
    	$rss_path = $this->config["proj_path"]."media/rss/cat".$catID.".xml";
    	if(file_exists($rss_path)){
    		unlink($rss_path);
    	}
    	
    	return true;
    }
    
    
}
?>