<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/


/**
 * Created on 17.04.2005
 * 
 * This class provides some functions for getting and setting
 * comments.
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 0.1
 */
 
class CommentManager {
	
	var $dbCon;
	var $config;
	var $options;
	
    function CommentManager($dbCon, $config, $options) {
    	$this->dbCon = $dbCon;
    	$this->config = $config;
    	$this->options = $options;
    }
    
    /**
     * Gets all comments of an item, ordered by date.
     * The important thing about this method is, that it
     * performs an LEFT OUTER JOIN on the comment table.
     * So you get all comments written by users or
     * by guests.
     * 
     * @param $itemID ID of an item
     * @param $offset the offset of the first row to return
     * @param $num The maximum number of rows to return
     * @return returns an array containing the comments
     */
    function getCommentsByItemID($itemID, $offset = 0, $num = 10) {
    	$sql = "SELECT * FROM ".$this->config["db_prefix"]."comment a " .
    		   "LEFT OUTER JOIN ".$this->config["db_prefix"]."user b ON " .
    		   "a.userID = b.userID where itemID = ".$itemID." order by " .
    		   "comment_date desc limit $offset,$num";
    		   
    	$result = $this->dbCon->selectFromDB($sql);
    	$comments = array();
    	while ($row = mysql_fetch_array($result)) {
    		$comments[count($comments)] = $row;
    	}
    	if(count($comments) > 0) return $comments;
    	return false;
    }
    
    /**
     * Expects an array with comments and changes them to be displayed on the website.
     * Changes:
     * - A new Field "comment_date" will be included that displays the date in the
     *   form the admin wishes it to be displayed.
     * 
     * @param $comments An array containing the comments.
     * @return A changed array containing the comments and more.
     * 
     */
    function createDisplayComments($comments, $lang_main, $item_manager) {
     	$new_comments = array();
     	foreach($comments as $comment) {
     		$new_comment = array();
     		if($comment["userID"] != null) {
     			$new_comment["username"] = $comment["username"];
     			$new_comment["userID"] = $comment["userID"];
     		} else {
     			$new_comment["username"] = $comment["guest_name"];
     			$new_comment["guest"] = $lang_main["guest"];
     		}
     		$new_comment["commentID"] = $comment["commentID"];
     		$new_comment["comment_text"] = $comment["comment_text"];
     		$new_comment["email_comment"] = $comment["email_comment"];
     		
     		//if full names of days are in date_format they must be translated first
     		if(preg_match("/l/", $this->options["date_format"])){
	 			$new_comment["comment_date"] = $item_manager->getDateString($this->options["date_format"], $comment["comment_date"], $lang_main);
			} else $new_comment["comment_date"] = date($this->options["date_format"],$comment["comment_date"]);
     		
     		$new_comments[count($new_comments)] = $new_comment;
     	}
     	return $new_comments;
     }
     
     /**
      * Gets comments for an item and changes them.
      * Returns an array containing the comments or false
      * if no comments exist.
      * 
      * @param $itemID ID of an item
      * @param $offset the offset of the first row to return
      * @param $num The maximum number of rows to return
      * @return returns an array containing the comments or false
      */
     function getDisplayComments($itemID, $lang_main, $item_manager, $offset = 0, $num = 10) {
     	$comments = $this->getCommentsByItemID($itemID, $offset, $num);
     	if(!$comments) return array();
     	return $this->createDisplayComments($comments,$lang_main, $item_manager);
     }
     
     /**
      * Returns a single comment.
      * 
      * @param commentID The id of the comment.
      * @return the item or false if no item with the specific ID exists.
      */
     function getComment($commentID) {
     	$sql = "SELECT * FROM ".$this->config["db_prefix"]."comment " .
    		   "where commentID = ".$commentID;
        
    	$result = $this->dbCon->selectFromDB($sql);
    	while ($row = mysql_fetch_array($result)) {
    		return $row;
    	}
    	return false;
     }
    
    /**
      * Returns the number of comments for an item
      * @param $itemID the id of the item
      * @return returns the number of comments for an item or false if the item does not exists
      */
     function getCommentCount($itemID){
     	$sql = "Select count(*) as comment_count from ".$this->config["db_prefix"]."comment where itemID =".$itemID;
		$result = $this->dbCon->selectFromDB($sql);
		while($row = mysql_fetch_array($result)) return $row["comment_count"];
		return false;
     }
     
     /**
      * Returns the number of comments for an item
      * @param $itemID the id of the item
      * @return returns the number of comments for an item or false if the item does not exists
      */
     function getCommentAuthorEmail($commentID){
     	$sql = "Select email_comment from ".$this->config["db_prefix"]."comment where itemID =".$itemID;
		$result = $this->dbCon->selectFromDB($sql);
		while($row = mysql_fetch_array($result)) return $row["email_comment"];
		return false;
     }
    
    /**
     * Adds a user comment to an item.
     * 
     */
    function addComment($itemID,$text,$username,$userID = false, $email="") {
    	$text = htmlentities($text);
    	if(!$userID) $sql = "INSERT INTO ".$this->config["db_prefix"]."comment (itemID,comment_text,comment_date,guest_name, email_comment) VALUES(".$itemID.",'".$text."',".time().",'".$username."', '".$email."')";
    	else $sql = "INSERT INTO ".$this->config["db_prefix"]."comment (itemID,comment_text,comment_date,userID) VALUES(".$itemID.",'".$text."',".time().",".$userID.")";
    	return $this->dbCon->insertIntoDB($sql);
    }
    
    /**
     * Deletes all comment of a specific item.
     */
    function deleteCommentsByItemID($itemID) {
    	$sql = "DELETE FROM ".$this->config["db_prefix"]."comment WHERE itemID = ".$itemID;
    	return $this->dbCon->queryDB($sql);
    }
    
    /**
     * Deletes a comment.
     */
    function deleteComment($commentID) {
    	$sql = "DELETE FROM ".$this->config["db_prefix"]."comment WHERE commentID = ".$commentID;
    	return $this->dbCon->queryDB($sql);
    }
    
}
?>