<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/


 require_once("extension.inc");
 require_once("config.php");
 if(!isset($config)) {
	 header("Location: ./install/install.php");
 }
 require_once($config["proj_path"].'/classes/MySmarty.class.php');
 require_once($config["proj_path"].'/classes/SessionManager.class.php');
 require_once($config["proj_path"].'/classes/CategoryManager.class.php');
 require_once($config["proj_path"].'/classes/DBConManager.class.php');
 require_once($config["proj_path"].'/classes/UserManager.class.php');
 require_once($config["proj_path"].'/classes/RankManager.class.php');
 require_once($config["proj_path"].'/classes/ItemManager.class.php');

class DisplayView {
    var $my_smarty;
    var $dbCon;
    var $options;
    var $session_manager;
    var $user_manager;
    var $cat_manager;
    var $item_manager;
    var $rank_manager;
 	var $valid_user = false;
 	var $user;
 	var $lang_main, $lang_error, $lang_encoding;
    
    function DisplayView() {
    	global $config;
    	$this->my_smarty = new MySmarty();
 		$this->session_manager = new SessionManager();
    	$this->dbCon = new DBConManager($config);
    	$this->options = $this->dbCon->getOptionsArray();
 		$this->user_manager = new UserManager($this->dbCon,$config);
 		$this->cat_manager = new CategoryManager($this->dbCon,$config);
 		$this->rank_manager = new RankManager($this->dbCon,$config);
     	$this->item_manager = new ItemManager($this->dbCon,$config,$this->options);
    	
    	// load the languages
    	require("./language/".$this->options["language"]."/lang_main.php");
		require("./language/".$this->options["language"]."/lang_error.php");
		require("./language/".$this->options["language"]."/lang.desc.php");
		
		$this->lang_main = $lang_main;
		$this->lang_error = $lang_error;
		$this->lang_encoding = $lang_encoding;
    	
 		if($this->session_manager->loggedIn()) {
		   $this->valid_user = $this->user_manager->isValidUser($this->session_manager->getUsername(),$this->session_manager->getUserpass());
		   if($this->valid_user) $this->user = $this->user_manager->getUser($this->session_manager->getUserID());
		}
		
		$this->createHeader();
    }
    
    /**
     * Sets the title of the webpage.
     */
    function setTitle($title) {
    	$this->my_smarty->assign("title", $title);
    }
    
    function showError($error_message) {
	   $this->setTitle($this->lang_error["error_title"]);
	   $this->my_smarty->assign('error_title',$this->lang_error["error_title"]);
	   $this->my_smarty->assign('error_message',$error_message);
	}
	
	function showErrorPage($error_message) {
	   $this->setTitle($this->lang_error["error_title"]);
	   $this->my_smarty->assign('error_title',$this->lang_error["error_title"]);
	   $this->my_smarty->assign('error_message',$error_message);
	   $this->my_smarty->display($this->options["template"].'/error.tpl');
	}
	
	function showSuccess($success_message) {
		$this->setTitle($this->lang_main["success_title"]);
	 	$this->my_smarty->assign('success_title',$this->lang_main["success_title"]);
		$this->my_smarty->assign('success_message',$success_message);
	}
	
	function showSuccessPage($success_message) {
	   $this->setTitle($this->lang_main["success_title"]);
	   $this->my_smarty->assign('success_title',$this->lang_main["success_title"]);
	   $this->my_smarty->assign('success_message',$success_message);
	   $this->my_smarty->display($this->options["template"].'/success.tpl');
	}
	
	function createHeader() {
     	global $config;
     	global $php_extension;
     	
    	$this->my_smarty->assign("encoding", $this->lang_encoding);
    	$this->my_smarty->assign("php_extension", $php_extension);
 		
 		$this->createMenuStructure();
 		$this->createFastAdmin();
 		$this->createLoginForm();
 		$this->displayMenuCategories();
     }
     
     /**
      * Creates the fast admin.
      */
     function createFastAdmin() {
     	global $config;
     	if(!$this->valid_user) return;
     	
     	$this->my_smarty->assign("fast_admin_title", $this->lang_main["navigation_title_fast_admin"]);
     	$this->my_smarty->assign("log_out", $this->lang_main["navigation_log_out"]);
     	
     	// Check, if the user may delete the item.
     	if(isset($_REQUEST["itemID"]) && $_REQUEST["itemID"] != "") {
     		$item = $this->item_manager->getItemByID($_REQUEST["itemID"]);
     		
     		if($this->rank_manager->editItemAllowed($this->user,$item)) {
     		   $this->my_smarty->assign("edit_item_allowed", "true");
     		   $this->my_smarty->assign("edit_item", $this->lang_main["edit_item"]);
     		}
     		
     		if($this->rank_manager->deleteItemAllowed($this->user,$item)) {
     		   $this->my_smarty->assign("delete_item_allowed", "true");
     		   $this->my_smarty->assign("delete_item", $this->lang_main["delete_item"]);
     		}
     		
     		if(isset($_REQUEST["showcomments"]) && $_REQUEST["showcomments"] == "true") {
     		   if($this->rank_manager->deleteCommentsAllowed($this->user,$item)) {
     		      $this->my_smarty->assign("delete_comments_allowed", "true");
     		   	  $this->my_smarty->assign("delete_all_comments", $this->lang_main["delete_all_comments"]);
     		   }
     		}
     	}
     	//Check if the user may delete a category
     	if($this->rank_manager->deleteCategoryAllowed($this->user)) {
	    	if(isset($_REQUEST["catID"]) && $this->cat_manager->isExistingCategory($_REQUEST["catID"])){
	 		   	$this->my_smarty->assign("delete_category_allowed", "true");
	 		    $this->my_smarty->assign("delete_category", $this->lang_main["delete_category"]);
	 		   	$this->my_smarty->assign("deleteID", $_REQUEST["catID"]);
 		    }
 		}
 		//Check if the user may edit a category
     	if($this->rank_manager->editCategoryAllowed($this->user)) {
	    	if(isset($_REQUEST["catID"]) && $this->cat_manager->isExistingCategory($_REQUEST["catID"])){
	 		   	$this->my_smarty->assign("edit_category_allowed", "true");
	 		    $this->my_smarty->assign("edit_category", $this->lang_main["edit_category"]);
	 		   	$this->my_smarty->assign("catID", $_REQUEST["catID"]);
 		    }
 		}
     	
     	$this->my_smarty->assign("logged_in_as", $this->lang_main["logged_in_as"]);
     }
     
     /**
      * Shows the login form.
      */
     function createLoginForm() {
     	if($this->valid_user) {
     		$this->my_smarty->assign('logged_in','true');
     		$this->my_smarty->assign('user_name',$this->session_manager->getUsername());
     		return;
     	}
     	
		$this->my_smarty->assign('mini_log_ref',$this->createSelfURL());
		$this->my_smarty->assign('username_label',$this->lang_main["username_label"]);
		$this->my_smarty->assign('password_label',$this->lang_main["password_label"]);
     }
     
     /**
      * Tries to create the valid url of the shown webpage.
      * This is important for the login procedure that will lead you
      * back to the page that you came from.
      * 
      * @return The URL of the shown webpage or login.php if it could not be retrieved.
      */
     function createSelfURL() {
     	if(isset($_SERVER["REQUEST_URI"])) return $_SERVER["REQUEST_URI"];
     	
     	$get_block = "";
 		$first = true;
     	foreach($_GET as $key => $value) {
 			($first == true) ? $get_block = $get_block."?".$key."=".$value : $get_block = $get_block."&amp;".$key."=".$value;
 			 $first = false;
 		}
 		
     	if(isset($_SERVER["PHP_SELF"])) {
     		$url = $_SERVER["PHP_SELF"];
     		return $url.$get_block;
     	} 
     	
     	return $this->getScriptName().$get_block;
     }
     
     /**
      * Creates the basic menu structure with its different points
      */
     function createMenuStructure() {
     	$this->my_smarty->assign('navigation_title_category',$this->lang_main["navigation_title_category"]);
		$this->my_smarty->assign('navigation_title_menu',$this->lang_main["navigation_title_menu"]);
		$this->my_smarty->assign('navigation_title_login',$this->lang_main["navigation_title_login"]);
//		$this->my_smarty->assign('navigation_howto',$this->lang_main["navigation_howto"]);
		
 		if(!$this->valid_user && $this->options["registration_allowed"] == 1) {
 			$this->my_smarty->assign('register',$this->lang_main["navigation_register"]);
 		}
 		if(!$this->valid_user) {
 			$this->my_smarty->assign('login_menu_label',$this->lang_main["navigation_login"]);
 		}
		if($this->valid_user && $this->rank_manager->addItemAllowed($this->user) && $this->cat_manager->getNumberOfCategories() > 0) {
 			$this->my_smarty->assign('create_item',$this->lang_main["navigation_create_item"]);
 		}
 		if($this->valid_user && $this->rank_manager->createCategoryAllowed($this->user)) {
 			$this->my_smarty->assign('create_category',$this->lang_main["navigation_create_category"]);
 		}
 		if($this->valid_user) $this->my_smarty->assign('navigation_edit_user_data',$this->lang_main["navigation_edit_user_data"]);
 		
 		if($this->valid_user && $this->rank_manager->isAdmin($this->user)) {
	       $this->my_smarty->assign("is_admin", "true");
	       $this->my_smarty->assign('admin_title_menu',$this->lang_main["admin_title_menu"]);
	       $this->my_smarty->assign('admin_area',$this->lang_main["admin_area"]);
	    }
	    
	    if($this->valid_user && $this->existsDownload()) {
	       $this->my_smarty->assign("download_on", "true");
 		   $this->my_smarty->assign('navigation_download',$this->lang_main["navigation_download"]);
	    }
     }
     
     /**
      * Tests if there is a download file for the mobile reporter midlet.
      * 
      * @return true if there is a file, else false
      */
     function existsDownload() {
     	if(file_exists("./download/mrep.zip"))
     		return true;
     	return false;
     }
     
     /**
      * Creates the category menue.
      */
     function displayMenuCategories() {
    	$categories = $this->cat_manager->getAllCategoriesOrderedByCount();
    	if(!$categories) $display_cat_count = 0;
    	else $display_cat_count = count($categories);
 		$total_cat_count = $this->cat_manager->getNumberOfCategories();
 		
    	$this->my_smarty->assign("nav_categories", $categories);
    	$this->my_smarty->assign("cat_more_button", $this->lang_main["navigation_more_button"]);
    	$this->my_smarty->assign("display_cat_count", $display_cat_count);
    	$this->my_smarty->assign("total_cat_count", $total_cat_count);
     }
     
     
//    /**
//     * This method is called, if the webpage that is represented by this
//     * class should be displayed.
//     */
//    abstract public function display();
//    
//    /**
//     * This method returns the name of the script that shows the webpage.
//     * It must be implemented by any viewable class just for the case, that
//     * a server does not support the variable $_SERVER["REQUEST_URI"]
//     */
//    abstract protected function getScriptName();
}
?>