<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/


/**
 * Created on 17.04.2005
 *
 * This class creates a RSS feed from a category
 *
 * To use this class, it is required, that a connection
 * to the I-Reporter Database has already been established.
 *
 * @author Alexander De Luca, Michael Mueller
 * @version 0.1
 */

class FeedCreator {

	var $dbCon;
    var $config;

    function FeedCreator($dbCon,$config) {
    	$this->dbCon = $dbCon;
    	$this->config = $config;
    }

    /**
     * Creates and shows a RSS Feed containing the items of a specific category
     * or, if $cat_id = false, of the newest items.
     *
     * @return The String representation of the RSS file.
     */
    function createRSSFeed($catID = false, $url = "./") {
    	global $config;
    	require_once($this->config["proj_path"].'classes/ItemManager.class.php');
    	require_once($this->config["proj_path"].'classes/MySmarty.class.php');
    	require_once($this->config["proj_path"].'classes/CategoryManager.class.php');
        require_once($this->config["proj_path"].'classes/SpecialSyntaxParser.class.php');
        require_once($this->config["proj_path"]."extension.inc");

    	$options = $this->dbCon->getOptionsArray();
    	require($url."/language/".$options["language"]."/lang.desc.php");
    	require($url."/language/".$options["language"]."/lang_main.php");

    	$options = $this->dbCon->getOptionsArray();
    	$cat_manager = new CategoryManager($this->dbCon, $this->config);
    	$item_manager = new ItemManager($this->dbCon,$this->config,$options);
    	$special_syntax_parser = new SpecialSyntaxParser($this->dbCon,$this->config);
    	$my_smarty = new MySmarty();

    	$my_smarty->assign('rss_lang_encoding',$lang_encoding);

    	if(!$catID) { // if no cat has been submitted
    		$my_smarty->display($options["template"].'/xml/rss.tpl');
    		return;
    	}

    	// else create Feed for a specific category

    	// get category data and include them in the template
    	$cat_data = $cat_manager->getCatByID($catID);
    	if(!$cat_data) return false;
    	$my_smarty->assign('channel_title',$cat_data["catname"]);
    	$my_smarty->assign('channel_link',$this->config["proj_url"]."feed.".$php_extension."?id=".$catID);
    	$my_smarty->assign('channel_pubDate',time());
    	$my_smarty->assign('channel_lastBuildDate',time());
    	$my_smarty->assign('channel_category',$cat_data["catname"]);

    	// now get the items for the channel, format and assign them to the template
    	$items = $item_manager->getAllItemsByCategory($catID,0,$cat_data["rss_items_count"]);
    	$items = $item_manager->createRSSItems($items,$special_syntax_parser, $lang_main);

    	$my_smarty->assign('item_url',$this->config["proj_url"]."item.".$php_extension."?itemID=");
    	$my_smarty->assign('items',$items);

    	// show the rss feed
    	$rss = $my_smarty->fetch($options["template"].'/xml/rss.tpl');

    	return $rss;
    }

    /**
     * Creates a RSS file for a category.
     */
    function createrRSSFile($catID) {
    	$rss_path = $this->config["proj_path"]."media/rss/";
    	if(!file_exists($rss_path))
    	  mkdir($rss_path);

    	$rss_data = $this->createRSSFeed($catID);

    	if($rss_data!=false) {
    		$rss_file = $rss_path."cat".$catID.".xml";
    		$temp_fp = fopen($rss_file, "w");
    		fputs($temp_fp, $rss_data);
			fclose($temp_fp);
    	}
    }
}
?>