<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/

/**
 * Created on 06.06.2005
 *
 * This class provides functions for handling files and folders,
 * to find languages or template files ...
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 0.1
 */
 
 
 class FolderManager {
 
 var $img_formats;
 
 /**
  * This function reads all the files and folders in a directory and returns
  * valid language folders.
  * 
  * @param folder the folder you want to check for language folders
  * @param $languages_found an array, that will containing the language
  * descriptions
  */
 function listLanguages($folder,&$languages_found) { // <- the & says, that a reference will be used
    $this->img_formats = array(0 => ".gif", 1 => ".jpeg", 2 => ".jpg", 3 => ".png");
 	$handle=opendir($folder);
	while (($file = readdir($handle))!=false) {
	   if ($file != "." && $file != "..") {
		   	if(is_dir($folder."/".$file) && preg_match('/lang(.*)/', $file)) {
		   		$this->listLanguages($folder."/".$file,$languages_found);
		   	} else if($file=="lang.desc.php") {
		   		include($folder."/".$file);
		   		$languages_found[count($languages_found)] = array("lang_short"=>$lang_short,"lang_encoding"=>$lang_encoding,"lang_description"=>$lang_description,"lang_folder"=>substr(strrchr($folder, "/"), 1));
		   	}
	   }
	}
	closedir($handle);
 }
 
 /**
  * This function reads all the files and folders in a directory and returns
  * valid template folders.
  * 
  * @param folder the folder you want to check for template folders
  * @param $templates_found an array, that will containing the template
  * descriptions
  */
 function listTemplates($folder,&$templates_found) { // <- the & says, that a reference will be used
 	$handle=opendir($folder);
	while (($file = readdir($handle))!=false) {
	   if ($file != "." && $file != ".." && is_dir($folder."/".$file)) {
	   	    $handle2=opendir($folder."/".$file);
	   	    while (($file2 = readdir($handle2))!=false) {
	   	    	if($file2=="temp.desc.php") {
			   		include($folder."/".$file."/".$file2);
			   		$templates_found[count($templates_found)] = array("template_name"=>$template_name,"template_author"=>$template_author,"template_description"=>$template_description,"template_folder"=>$file);
			   	}
	   	    }
	   	    closedir($handle2);
	   }
	}
	closedir($handle);
 }
 
 /**
  * This function reads all the mail template files and folders in a directory and returns
  * valid mail templates.
  * 
  * @param folder the folder you want to check for mail tempaltes
  * @param $templates_found an array, that will containing the mail data
  */
 function listMailTemplates($folder,&$templates_found) {
 	$handle=opendir($folder);
	while (($file = readdir($handle))!=false) {
	   if ($file != "." && $file != ".." && !is_dir($folder."/".$file)) {
	   		include($folder."/".$file);
	   		if(isset($mail_name))
	   			$templates_found[count($templates_found)] = array("name"=>$mail_name,"mail_author"=>$mail_author,"mail_server"=>$mail_server,"mail_protocol"=>$mail_protocol,"mail_port"=>$mail_port,"mail_file"=>$file);
	   }
	}
	usort($templates_found, array("FolderManager", "cmp"));
	closedir($handle);
 }
 
  /**
  * This function scans a folder and returns an array containing all files it found
  * with their filesize, ordered by the filename.
  * 
  * Important: The list doesn't include folders.
  * 
  * @param path the path to the folder that has to be checked
  */
 function listFilesByFolder($path) {
 	$files = array();
 	if(!file_exists($path)) return $files;
 	$handle=opendir($path);
	while (($file = readdir($handle))!=false) {
	   if ($file != "." && $file != ".." && !is_dir($path.$file)) {
	   	  $place = count($files);
   		  $files[$place]["name"] = $file;
   		  
   		  if(function_exists("getimagesize")) {
   		  	$imginfo = getimagesize($path.$file);
//   		  	print_r($imginfo);
   		  	if($imginfo != false && $imginfo[2] >= 1 && $imginfo[2] <= 3)
   		  	  $files[$place]["image"] = $imginfo[2];
   		  } else { // try to match the file names
   		    $low_string = strtolower($file);
   		  	foreach($this->img_formats as $format) {
   		  		$pos = strripos($low_string,$format);
   		  		if($pos != false) {
   		  		  if(strlen($low_string) == $pos+strlen($format)) {
   		  		  	$files[$place]["image"] = 1;
   		  		  	break;
   		  		  }
   		  		}
   		  	}
   		  }
   		  if(function_exists("filesize"))
   		     $files[$place]["size"] = filesize($path.$file);
	   }
	}
	closedir($handle);
//	usort($files, "cmp");
	usort($files, array("FolderManager", "cmp"));
//	sort($files);
	
	return $files;
 }
 
 function cmp($a, $b) {
    return strcmp($a["name"], $b["name"]);
 }
 
}


?>
