<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/


/**
 * Created on 17.04.2005
 *
 * This class provides some functions for getting items out of the db
 * and returns the parameters of one or more items in an array.
 *
 * To use this class, it is required, that a connection
 * to the I-Reporter Database has already been established.
 *
 * @author Alexander De Luca, Michael Mueller
 * @version 0.2
 */

class ItemManager {

	var $dbCon;
	var $config;
	var $options;

    function ItemManager($dbCon, $config, $options) {
    	$this->dbCon = $dbCon;
    	$this->config = $config;
    	$this->options = $options;
    }

    /**
     * Gets an item by its id and returns an array
     *
     * @param $itemID ID of an item
     * @param $show_invisible true if also invisible items shall be shown.
     * @return $item returns an array if the specified itemID exists, if not it returns null
     */
    function getItemByID($itemID,$show_invisible = false){
    	$sql = "Select * from ".$this->config["db_prefix"]."item natural join " .
    			"".$this->config["db_prefix"]."user where itemID = ".$itemID." and item_visible = 1";
    	if($show_invisible)
    	    $sql = "Select * from ".$this->config["db_prefix"]."item natural join " .
    			"".$this->config["db_prefix"]."user where itemID = ".$itemID;

    	$result = $this->dbCon->selectFromDB($sql);
    	while ($array = mysql_fetch_array($result)){
    		return $array;
    	}
    	return false;
    }

    /**
     * Deletes all items, that are invisible and older then a specific time.
     */
    function deleteUnusedItems() {
    	$sql = "SELECT * FROM ".$this->config["db_prefix"]."item WHERE item_visible = 0 and item_timestamp <= ".(time()-600);
    	$result = $this->dbCon->selectFromDB($sql);

    	while($row = mysql_fetch_array($result)) {
    		$this->deleteItem($row["itemID"]);
    	}
    }

    /**
     * Deletes every part of an item in the database.
     */
    function deleteItem($itemID) {
    	$sql = "SELECT * FROM ".$this->config["db_prefix"]."item WHERE itemID = ".$itemID;
    	$result = $this->dbCon->selectFromDB($sql);

    	$item_path = $this->config["proj_path"]."media";
    	while($row = mysql_fetch_array($result)) {
    		$item_path = $item_path."/cat".$row["catID"];
    		$catID = $row["catID"];
    	}
    	$item_path = $item_path."/item".$itemID;

    	$sqls = array(
    	"DELETE FROM ".$this->config["db_prefix"]."item WHERE itemID = ".$itemID,
    	"DELETE FROM ".$this->config["db_prefix"]."comment WHERE itemID = ".$itemID,
    	"DELETE FROM ".$this->config["db_prefix"]."item_view WHERE itemID = ".$itemID,
    	"DELETE FROM ".$this->config["db_prefix"]."item_visit WHERE itemID = ".$itemID,
    	"DELETE FROM ".$this->config["db_prefix"]."links WHERE from_item = ".$itemID." or to_item = ".$itemID,
    	"DELETE FROM ".$this->config["db_prefix"]."rating WHERE itemID = ".$itemID
    	);

    	foreach($sqls as $sql) {
    		if(!$this->dbCon->queryDB($sql)) return false;
    	}

    	$sql = "SELECT * FROM ".$this->config["db_prefix"]."mediadata WHERE itemID = ".$itemID;
    	$result = $this->dbCon->selectFromDB($sql);

    	while($row = mysql_fetch_array($result)) {
    		if(file_exists($row["url"]))
    		   unlink($row["url"]);
    	}
    	// delete thumb folder
    	$thumb_path = $item_path."/thumb";
    	if(file_exists($thumb_path)) {
    		$handle=opendir($thumb_path);
			while (($file = readdir($handle))!=false) {
			   if ($file != "." && $file != ".." && !is_dir($file)) {
		   		  if(file_exists($thumb_path."/".$file))
		   		     unlink($thumb_path."/".$file);
			   }
			}
			closedir($handle);
			rmdir($thumb_path);
    	}
    	if(file_exists($item_path)) rmdir($item_path);

    	$sql = "DELETE FROM ".$this->config["db_prefix"]."mediadata WHERE itemID = ".$itemID;
    	if(!$this->dbCon->queryDB($sql)) return false;

    	return true;
    }

    /**
     * Deletes a media by its filename and item.
     */
    function deleteMedia($filename, $item) {
    	global $config;
    	$sql = "DELETE FROM ".$this->config["db_prefix"]."mediadata WHERE itemID = ".$item["itemID"]." and filename = '".$filename."'";
    	$user_path = $config["proj_path"]."media/cat".$item["catID"]."/item".$item["itemID"]."/";

    	$thumb_folder = $user_path."thumb/";
    	if(file_exists($thumb_folder."tn_".$filename)) {
    		unlink($thumb_folder."tn_".$filename);
    	}

    	if(file_exists($user_path.$filename)) {
    		unlink($user_path.$filename);
    	}

    	if(!$this->dbCon->queryDB($sql)) return false;
    	return true;
    }


    /**
     * Gets all the items by the same user with specified userID
     *
     * @param $userID the userID
     * @param $offset the offset of the first row to return
     * @param $num the maximum number of rows to return
     * @return $items if existing user: an array with as many items in it as given by $num,
     *                else null
     */
    function getAllItemsByUserID($userID, $offset=0, $num=10){

    	$sql = "Select * from ".$this->config["db_prefix"]."item natural join ".$this->config["db_prefix"]."user where userID = ".$userID." and item_visible = 1 order by item_timestamp desc, itemID desc limit $offset,$num";
    	$result = $this->dbCon->selectFromDB($sql);
    	$items = array();
    	while ($array = mysql_fetch_array($result)){
    		$items[count($items)] = $array;
    	}
    	if (count($items) > 0) return $items;
    	return false;
    }

    /**
     * Gets all the items of a single category
     *
     * @param $catID ID of the category
     * @param $offset the offset of the first row to return
     * @param $num The maximum number of rows to return
     * @return $items Array with max. as many items in it as given by $num
     */
     function getAllItemsByCategory($catID, $offset = 0, $num = 10){

    	if($num == "all"){
    		$sql = "Select * from ".$this->config["db_prefix"]."item where catID = ".$catID;
    	}
    	else $sql = "Select * from ".$this->config["db_prefix"]."item natural join ".$this->config["db_prefix"]."user where catID = ".$catID." and item_visible = 1 order by item_timestamp desc, itemID desc limit $offset,$num";
    	$result = $this->dbCon->selectFromDB($sql);
    	$items = array();
    	while ($array = mysql_fetch_array($result)){
    		$items[count($items)] = $array;
    	}
    	if (count($items) > 0) return $items;
    	return false;
     }

     /**
      * Returns the number of items for a category
      * @param $catID the id of a category
      * @return returns the number of items for a category
      */
     function getItemCount($catID){
     	$sql = "Select count(*) as item_count from ".$this->config["db_prefix"]."item where catID =".$catID." and item_visible = 1";
		$result = $this->dbCon->selectFromDB($sql);
		while($row = mysql_fetch_array($result)) return $row["item_count"];
		return false;
     }

    /**
     * Gets all items
     *
     * @param $offset the offset of the first row to return
     * @param $num The maximum number of rows to return
     * @return $items Array with max. as many items in it as given by $num
     */
     function getAllItems($offset = 0, $num = 10){

    	$sql = "Select * from ".$this->config["db_prefix"]."item natural join ".$this->config["db_prefix"]."user where item_visible = 1 order by item_timestamp desc, itemID desc limit $offset,$num";
    	$result = $this->dbCon->selectFromDB($sql);
    	$items = array();
    	while ($array = mysql_fetch_array($result)){
    		$items[count($items)] = $array;
    	}
    	if (count($items) > 0) return $items;
    	return false;
     }


     /**
      * Gets a number of items (except the most actual one) with the highest total_factor which is computed by
      * <ul>
      *   <li>a view visitor factor</li>
      *   <li>a view user factor</li>
      *   <li>the age of the item (in days)</li>
      * </ul>
      * @param $item_view_user_factor number between 0 and 1 to weight the visit of an user
      * @param $item_view_visitor_factor number between 0 and 1 to weight the visit of a visitor
      * @param $act_itemID the id of the most actual item
      */
     function getMostViewedItems($item_view_user_factor, $item_view_visitor_factor, $act_itemID) {
     	

     	$sql = "SELECT * , ("
     	        ."SELECT (((sum( visitor_count ) * ".$item_view_visitor_factor.") + ( sum( user_count ) * ".$item_view_user_factor.")) / ((UNIX_TIMESTAMP( ) - i.item_timestamp ) /86400))FROM ".$this->config["db_prefix"]."item_view v WHERE v.itemID = i.itemID"
     	        .") AS total_factor"
				." FROM ".$this->config["db_prefix"]."item i, ".$this->config["db_prefix"]."user u"
				." WHERE i.userID = u.userID"
				." AND i.itemID !=".$act_itemID
				." ORDER BY total_factor DESC LIMIT 10";

		$result = $this->dbCon->selectFromDB($sql);
		$num_rows = mysql_num_rows($result);
		$most_viewed_items = array();
		while($row = mysql_fetch_assoc($result)){
			$most_viewed_items[count($most_viewed_items)] = $row;
		}
		if($num_rows > 0) return $most_viewed_items;
		return null;
     }

     /**
      * This function gets the most actual item and returns all its data
      */
     function getMostActualItem($special_syntax_parser,$comment_manager,$thumb_creator, $lang_main, $item_view_user_factor, $item_view_visitor_factor){
     	$sql2 = "select * from ".$this->config["db_prefix"]."item i, ".$this->config["db_prefix"]."user u "
     		  ." where i.userID = u.userID order by i.item_timestamp desc limit 1";
     	$result = $this->dbCon->selectFromDB($sql2);
     	while ($row = mysql_fetch_assoc($result)){
			return $row;
     	}
     	return null;
     }

     /**
      * This function checks whether a user already rated an item or not
      * @param $userID the userID of the user logged in
      * @param $itemID the item that the user views
      * @return true if the user has not yet rated this item, else false
      */
     function checkItemRating($userID, $itemID){
     	$sql = "select count(*) as rating_count from ".$this->config["db_prefix"]."rating where userID = $userID and itemID = $itemID";
     	$result = $this->dbCon->selectFromDB($sql);
     	while($row = mysql_fetch_assoc($result)){
     		if($row["rating_count"] == 0) return true;
     		return false;
     	}
     }

     /**
      * Sets an item visible or invisible.
      *
      * @param itemID The ID of the item.
      * @param set_visible true if it shall be visible, else false
      * @return true if everything went right, else false
      */
     function setVisible($itemID,$set_visible) {
     	$item_visible = 0;
     	if($set_visible) $item_visible = 1;
     	$sql = "Update ".$this->config["db_prefix"]."item set item_visible = ".$item_visible." where itemID = ".$itemID;
    	$result = $this->dbCon->queryDB($sql);
    	return $result;
     }

     /**
      * Sets the main img for an item.
      *
      * @param $itemID The ID of the item.
      * @param $mediaID The ID of the media file that is the main_img
      */
     function setMainImg($itemID,$mediaID) {
     	$sql = "Update ".$this->config["db_prefix"]."item set main_img = ".$mediaID." where itemID = ".$itemID;
    	$result = $this->dbCon->queryDB($sql);
    	return $result;
     }

     /**
      * Changes the text of an item.
      */
     function updateItemText($itemID,$newText) {
     	$in_text = str_replace("'", "\'", $newText);
     	$sql = "Update ".$this->config["db_prefix"]."item set item_text = '".$in_text."' where itemID = ".$itemID;
    	$result = $this->dbCon->queryDB($sql);
    	return $result;
     }

     /**
      * Expects an array with items and changes them to be displayed on the
      * category overview page.
      * Changes:
      * - A new Field "item_date" will be included that displays the date in the
      *   form the admin wishes it to be displayed.
      * - Adds an media array to each item. For items without media this will be
      *   an array with size 0.
      * - The text will be changed with the special syntax parser and all images
      *   will be removed.
      *
      * @param $items An array containing the items.
      * @return A changed array containing the items and more.
      */
     function createDisplayItems($items,$special_syntax_parser,$comment_manager,$thumb_creator, $lang_main) {
     	$new_items = array();
     	foreach($items as $item) {
     		if(preg_match("/l/", $this->options["date_format"])){
	 			$item["item_date"] = $this->getDateString($this->options["date_format"], $item["item_timestamp"], $lang_main);
			    $item["split_date"] = $this->getDateString($this->options["split_item_date_format"],$item["item_timestamp"], $lang_main);
			} else {
				$item["item_date"] = date($this->options["date_format"],$item["item_timestamp"]);
				$item["split_date"] = date($this->options["split_item_date_format"],$item["item_timestamp"]);
     		}

     		$item["item_media"] = $this->getMediaByItem($item["itemID"]);
     		$item["item_text"] = $special_syntax_parser->parseRuntimeSpecialSyntax($item["item_text"],$item["itemID"],false);
     		if($this->options["adjust_overview"] == 1) {
     			$item["item_text"] = strip_tags($item["item_text"], "<br><a>");
     			if(count($item["item_media"] > 0)){
     				$thumbnail = $thumb_creator->createThumbTag($item,$this);
  		   			if($thumbnail!=false)
     				   $item["main_img"] = $thumbnail;
     			}
 			}
     		$item["comments_count"] = $comment_manager->getCommentCount($item["itemID"]);
     		$new_items[count($new_items)] = $item;
     	}
     	if(count($new_items) > 0) return $new_items;
     	else return null;
     }

     /**
      * Does the important changes to display items in the RSS file.
      * For example it removes all html tags of the text and it changes the date of the items.
      */
     function createRSSItems($items, $special_syntax_parser, $lang_main) {
     	$new_items = array();
     	foreach($items as $item) {
     		if(preg_match("/l/", $this->options["date_format"])){
	 			$item["item_date"] = $this->getDateString($this->options["date_format"], $item["item_timestamp"], $lang_main);
			} else $item["item_date"] = date($this->options["date_format"],$item["item_timestamp"]);
     		$item["item_text"] = $special_syntax_parser->parseRuntimeSpecialSyntax($item["item_text"],$item["itemID"],false);
     		$item["item_text"] = strip_tags($item["item_text"]);
     		$item["item_text"] = html_entity_decode($item["item_text"]);

     		$new_items[count($new_items)] = $item;
     	}
     	return $new_items;
     }


     function createSingleDisplayItem($item,$special_syntax_parser,$comment_manager,$thumb_creator, $lang_main) {
 		if(preg_match("/l/", $this->options["date_format"])){
 			$item["item_date"] = $this->getDateString($this->options["date_format"], $item["item_timestamp"], $lang_main);
		    $item["split_date"] = $this->getDateString($this->options["split_item_date_format"],$item["item_timestamp"], $lang_main);
		} else {
			$item["item_date"] = date($this->options["date_format"],$item["item_timestamp"]);
			$item["split_date"] = date($this->options["split_item_date_format"],$item["item_timestamp"]);
 		}
 		$item["item_media"] = $this->getMediaByItem($item["itemID"]);
 		$item["item_text"] = $special_syntax_parser->parseRuntimeSpecialSyntax($item["item_text"],$item["itemID"],false);
 		
 		if($this->options["adjust_overview"] == 1) {
 			$item["item_text"] = strip_tags($item["item_text"], "<br><a>");
 			$item["comments_count"] = $comment_manager->getCommentCount($item["itemID"]);
 			if(count($item["item_media"] > 0)) {
 				$thumbnail = $thumb_creator->createThumbTag($item,$this);
 				if($thumbnail!=false)
 			   		$item["main_img"] = $thumbnail;
 			}
		}
 		if(count($item) > 0) return $item;
 		else return null;
     }


     /**
      * Returns an array containing all the media for an item.
      * It contains the urls of the files.
      *
      * @return An array containing all the media for an item.
      */
     function getMediaByItem($item_id) {
        $sql = "Select * from ".$this->config["db_prefix"]."mediadata where itemID = ".$item_id;
    	$result = $this->dbCon->selectFromDB($sql);
    	$media = array();
    	while ($array = mysql_fetch_array($result)){
    		$media[count($media)] = $array;
    	}
    	return $media;
     }

     /**
      * Returns a media data by id.
      *
      * @return A media data by id.
      */
     function getMediaByID($mediaID) {
        $sql = "Select * from ".$this->config["db_prefix"]."mediadata where mediaID = ".$mediaID;
    	$result = $this->dbCon->selectFromDB($sql);
    	while ($array = mysql_fetch_array($result)){
    		return $array;
    	}
    	return false;
     }

     /**
      * Adds a media to the database, if it is not already
      * registered for this item.
      */
     function addMediaToDB($itemID, $catID, $filename, $is_main) {
     	$sql = "Select * from ".$this->config["db_prefix"]."mediadata where filename = '".$filename."' and itemID = ".$itemID;
    	$result = $this->dbCon->selectFromDB($sql);
    	while ($array = mysql_fetch_array($result)){
    		if($is_main)
    	        $this->setMainImg($itemID,$array["mediaID"]);
    		return;
    	}

     	$url = "./media/cat".$catID."/item".$itemID."/".$filename;
    	$sql = "insert into ".$this->config["db_prefix"]."mediadata values (null,'$filename', '$url', $itemID)";
    	$success = $this->dbCon->insertIntoDB($sql);
    	if ($success) {
    		$mediaID = mysql_insert_id();
    		if($is_main)
    	       $this->setMainImg($itemID,$mediaID);
    	}

    	return $success;
     }

     function getRatingByItem($itemID){
     	$sql = "select * from ".$this->config["db_prefix"]."rating where itemID = ".$itemID;
     	$result = $this->dbCon->selectFromDB($sql);
     	$ranked_good = 0;
     	$ranked_bad = 0;
     	while($row = mysql_fetch_assoc($result)){
     		if($row["rating"] == 1) $ranked_good++;
     		else $ranked_bad++;
     	}
     	if($ranked_good > 0 || $ranked_bad > 0) return array($ranked_bad, $ranked_good);
     	return false;
     }

     /**
      * If the full name of the day of a week is set in date_format ("l") this function
      * creates a date string that matches with the language set in the options
      */
      function getDateString($date_format, $timestamp, $lang_main){
      	$date = date($date_format, $timestamp);
      	$day = date("l", $timestamp);
      	switch ($day){
      		case "Sunday": $date = str_replace("Sunday", $lang_main["sunday"], $date); break;
      		case "Monday": $date = str_replace("Monday", $lang_main["monday"], $date); break;
      		case "Tuesday": $date = str_replace("Tuesday", $lang_main["tuesday"], $date); break;
      		case "Wednesday": $date = str_replace("Wednesday", $lang_main["wednesday"], $date); break;
      		case "Thursday": $date = str_replace("Thursday", $lang_main["thursday"], $date); break;
      		case "Friday": $date = str_replace("Friday", $lang_main["friday"], $date); break;
      		case "Saturday": $date = str_replace("Saturday", $lang_main["saturday"], $date); break;
      	}
      	return $date;
      }
}
?>