<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/

/**
 * Created on 31.03.2005
 * 
 * This class is the one that finds out all the important
 * data for an item and writes it to the db.
 * 
 * To use this class, it is required, that a connection
 * to the I-Reporter Database has already been established.
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 0.1
 */


define("RTE_EDITOR",0);
define("WEBSITE_EDITOR",1);
define("MIDLET_EDITOR",2);
define("EMAIL_EDITOR",3);
define("OTHER_EDITOR",4);
class ItemParser {
	
	var $title;
	var $category;
	var $dbCon;
	var $config;
	var $cat_manager;
	var $thumb_creator;
	var $item_manager;
	var $feed_creator;
    
    function ItemParser($dbCon, $config) {
    	$this->dbCon = $dbCon;
    	$this->config = $config;
    	
    	require_once($this->config["proj_path"].'/classes/SpecialSyntaxParser.class.php');
    	require_once($this->config["proj_path"].'/classes/ItemManager.class.php');
    	require_once($this->config["proj_path"].'/classes/CategoryManager.class.php');
    	require_once($this->config["proj_path"].'/classes/FeedCreator.class.php');
    	require_once($this->config["proj_path"].'/classes/ThumbCreator.class.php');
    	$this->cat_manager = new CategoryManager($this->dbCon, $this->config);
    	$this->item_manager = new ItemManager($this->dbCon, $this->config,$this->dbCon->getOptionsArray());
    	$this->thumb_creator = new ThumbCreator($this->dbCon, $this->config, $this->dbCon->getOptionsArray());
    	$this->feed_creator = new FeedCreator($this->dbCon,$this->config);
    }
    
    /**
     * Analyses an array to find out the important data for an item.
     * The Array can contain: 
     * <ul>
     * <li>"user"</li>
     * <li>"timestamp"</li>
     * <li>"title"</li>
     * <li>"item_text"</li>
     * <li>"location"</li>
     * <li>"category"</li>
     * <li>Array->"media_data"</li>
     * </ul>						
     * 
     * @param $item_data An array that contains specific data
     * like user, title and so on, but doesn't have to implement
     * any of them.
     */
    function parseItem($item_data, $item_visible=1) {

		$userID = $this->getUserIDFromUserdata($item_data["user"]);
//    	echo "UserID: ".$userID." adresse: ".$item_data["user"];
    	if (!isset($item_data["timestamp"])) $timestamp = time();
    	else $timestamp = $item_data["timestamp"];
    	
    	if (isset($item_data["item_text"])){
    		if (preg_match("/\[([ctliaCTLIA].+)\]/", $item_data["item_text"])){
    			$specialSyntaxParser = new SpecialSyntaxParser($this->dbCon, $this->config);
    			$temp_array = $specialSyntaxParser->parseStaticSpecialSyntax($item_data["item_text"]);
    			
    			foreach($temp_array as $key => $value) {
				  $item_data[$key] = $value;	
				}
    		}
    		
			$item_text = $item_data["item_text"];
			$item_text = $this->filterHtmlFromItemText($item_text,$item_data["item_editor"]);
    	}
    	if (!isset($item_data["title"])){
    		if (preg_match("/\[([ctliaCTLIA].+)\]/", $item_text)){
    			$item_text = preg_replace("/(\[[ctliaCTLIA]):([^\[\]]+)\])+/", "", $item_text);
    		}
    		$title = substr($item_text, 0, $this->config["num_characters_for_title"]);
    	}
    	else $title = $item_data["title"];
    	
    	if (isset($item_data["category"])) $category = $this->cat_manager->isExistingCategory($item_data["category"]);
    	else $category = $this->cat_manager->getDefaultCatID();
    	
    	if (isset($item_data["location"])){
    		$location = "'".$item_data["location"]."'";
    	}
    	else $location = "NULL";
    	
    	if (isset($item_data["time_needed"])){
    		$creation_time = $item_data["time_needed"];
    	} else $creation_time = "NULL";
		
    	if (!$userID) {
    		return;
    	}
    	
    	// if an itemID is set, that means change an item
        if (isset($item_data["itemID"])){
        	//get old data
        	$old_item = $this->item_manager->getItemByID($item_data["itemID"]);
        	
        	$sql = "Update ".$this->config["db_prefix"]."item" .
        			" set title = '$title', catID = $category, location = $location, item_text = '$item_text'," .
        			" main_img = NULL, item_visible = $item_visible" .
        			" where itemID = ".$item_data["itemID"];
            $success = $this->dbCon->insertIntoDB($sql);
//            $this->feed_creator->createRSSFile($category);
//            if($category != $old_item["catID"])
//               $this->feed_creator->createRSSFile($old_item["catID"]);
        	return $item_data["itemID"];
        }
        
        // else add new item
    	$sql = "Insert INTO ".$this->config["db_prefix"]."item values (null, $timestamp, " .
    			"'$title', $userID, $category, " .
    			"$location, " .
    			"'$item_text',null, $creation_time,$item_visible)";
        $success = $this->dbCon->insertIntoDB($sql);
    	if ($success) {
    		$itemID = mysql_insert_id();
//            $this->feed_creator->createRSSFile($category);
    	}
    	else echo mysql_error();
    	
    	if (isset($item_data["media_data"]) && count($item_data["media_data"]) > 0){
    		$this->saveMediaData($item_data["media_data"], $itemID, $category);
    	}
    	return $itemID;
    }
    
    /**
     * Adds Media after the item was already inserted into the db
     * @param $itemID The ID of the item that was already inserted into the db
     * @param $media_data Array containing the filename and the data of each media file of this item
     */
    function addMedia($itemID, $media_data){
    	
    	$sql = "Select * from ".$this->config["db_prefix"]."item where itemID =".$itemID;
    	$result = $this->dbCon->selectFromDB($sql);
    	while ($result_array = mysql_fetch_array($result)){
    		$catID = $result_array["catID"];
    	}
    	
    	if (isset($catID)) {
    		$this->saveMediaData($media_data, $itemID, $catID);
    	} else
    	    sendError($lang_error["wrong_itemid_in_db"]);
    }
    
    /**
     * Tries to find an existing user, depending on any
     * String that represents parts of userdata.
     */
    function getUserIDFromUserdata($user_data) {
    	// is it an username?
    	$get_user_sql = "SELECT * FROM ".$this->config["db_prefix"]."user WHERE username = '".$user_data."'";
    	
    	$result = mysql_query($get_user_sql);
    	while($row = mysql_fetch_array($result)) {
    		return $row["userID"];
    	}
    	// is it an email address
    	$get_user_sql = "SELECT * FROM ".$this->config["db_prefix"]."user WHERE email = '".$user_data."'";
    	$result = mysql_query($get_user_sql);
    	while($row = mysql_fetch_array($result)) {
    		return $row["userID"];
    	}
    	// is it a mobile phone number
    	$mailbox = substr($user_data, 0, strpos($user_data, "@"));
    	if (preg_match("/[0-9]+/", $mailbox)){
    		//compare the two mobilephone numbers
    		$get_user_sql = "SELECT * FROM ".$this->config["db_prefix"]."user WHERE mobilephone_number = '".$mailbox."'";
	    	$result = mysql_query($get_user_sql);
	    	while($row = mysql_fetch_array($result)) {
	    		return $row["userID"];
	    	}
	    	//compare the mobilephone number from the email-address without the first two digits with the mobile-number of the user
	    	$mailbox = "0".substr($mailbox, 2);
//	    	echo "mailbox = ".$mailbox."<br/>";
	    	$get_user_sql = "SELECT * FROM ".$this->config["db_prefix"]."user WHERE mobilephone_number = '".$mailbox."'";
	    	$result = mysql_query($get_user_sql);
	    	while($row = mysql_fetch_array($result)) {
	    		return $row["userID"];
	    	}
    	}
    	return false;
    }
    
    
    /**
     * Saves all the elements of the media_data array (attachments) on the server side
     */
    function saveMediaData($media_data, $itemID, $catID){
    	
    	$cat_path = $this->config["proj_path"]."media/cat".$catID."/";
    	$path = $cat_path."item".$itemID."/";    	
    	
    	if(!file_exists($cat_path)){
    		mkdir($cat_path);
    	}
    	if(!file_exists($path)){
    		mkdir($path);
    	}
    	$media_count = count($media_data);
    	for ($i = 0; $i < $media_count; $i++){
    		$this->saveFile($media_data[$i],$path, $catID,$itemID);
    	}
    }
    
    function saveFile($media_data, $path, $catID, $itemID, $number = 0){
    	$options = $this->dbCon->getOptionsArray();
    	if($options["max_files"] >= 0) {
    		$media_count = count($this->item_manager->getMediaByItem($itemID));
    		if($media_count+1 > $options["max_files"])
    			return;
    	}
    	$filename = $media_data["name"];
    	$ending = "";
    	if($number > 0) $ending = $number;
		$file = substr($filename, 0, strrpos($filename, "."));
		$extension = strtolower(substr($filename, strrpos($filename, ".")+1));
		$filename = $file.$ending.".".$extension;
		
		if (!file_exists($path . $filename)) {
			$temp_file = $path . $filename;
			if(isset($media_data["tmp_name"])) {
				move_uploaded_file($media_data["tmp_name"],$temp_file);
			} else if(isset($media_data["user_tmp_name"])) {
				copy($media_data["user_tmp_name"],$temp_file);
			} else {
				$temp_fp = fopen($temp_file, "w");
				fputs($temp_fp, $media_data["data"]);
				fclose($temp_fp);
			}
		    
		    if($this->thumb_creator->hasGDLib()) { // try to create thumbnail
		    	$this->thumb_creator->createThumbnail($path,$filename);
		    }
		    
			$mediaID = $this->saveMediaToDB($filename, $catID, $itemID);
			
			if($mediaID != false && isset($media_data["main_img"]) && $media_data["main_img"] == "true") {
				$this->item_manager->setMainImg($itemID,$mediaID);
			}
			
		}
		else {  //filename already exists
		    $number++;
			$this->saveFile($media_data, $path, $catID,$itemID, $number);
		}
    }
    
    function saveMediaToDB($filename, $catID, $itemID) {
    	$url = "./media/cat".$catID."/item".$itemID."/".$filename;
    	$sql = "insert into ".$this->config["db_prefix"]."mediadata values (null,'$filename', '$url', $itemID)";
    	$success = $this->dbCon->insertIntoDB($sql);
    	if ($success) {
    		$itemID = mysql_insert_id();
    		return $itemID;
    	}
    	return false;
    }
    
    /**
     * Checks if html is allowed in options and if no, it converts 
     * the html special characters in the item_text in plain text
     * 
     * @param $item_text The text of an item
     * @param $item_editor The editor the item was created with.
     * @return $item_text returns the item_text
     */    
     function filterHtmlFromItemText($item_text,$item_editor){
    	$options = $this->dbCon->getOptionsArray();
    	if($item_editor == RTE_EDITOR)
    	   return $item_text;
    	$html_allowed = $options["html_allowed"];
    	if ($html_allowed == 0){
    		return htmlentities($item_text);
    	}
    	return $item_text;
    }
}
?>