<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/


/**
 * Created on 09.06.2005
 * 
 * This class does the rank management. It offers methods for adding
 * deleting, changing ranks and more.
 * 
 * A rank describes all the things a user is and is not allowed to do.
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 0.1
 */
 
class RankManager {
	
	var $dbCon;
	var $config;

   function RankManager($dbCon, $config) {
    	$this->dbCon = $dbCon;
    	$this->config = $config;
   }
    
  /**
   * Returns an array containing the rank or false if
   * no rank with this ID exists.
   */
  function getRankByID($rankID) {
  	$sql = "Select * from ".$this->config["db_prefix"]."rank where rankID=".$rankID;
  	$result = $this->dbCon->selectFromDB($sql);
  	while($row = mysql_fetch_array($result)){
  		return $row;
  	}
  	return false;
  }
  
  /**
   * Returns an array containing all ranks ordered by the name.
   * 
   * @param show_all Indicates if alle ranks will be shown or only the
   * visible ranks. Default is false.
   * @return An array containing alle ranks ordered by the name or false if an error occured.
   */
  function getAllRanks($show_all = false) {
  	$sql = "Select * from ".$this->config["db_prefix"]."rank where rank_visible = 1 order by rank_name";
  	if($show_all) $sql = "Select * from ".$this->config["db_prefix"]."rank order by rank_name";
  	$result = $this->dbCon->selectFromDB($sql);
  	if(!$result) return false;
  	$ranks = array();
  	while($row = mysql_fetch_array($result)){
  		$ranks[count($ranks)] = $row;
  	}
  	return $ranks;
  }
  
  /**
   * Returns true, if the user is allowed to delete the specified item,
   * else false.
   * 
   * @param user An array containing all user data.
   * @param item An array containing the item data.
   * @return true, if the user is allowed to delete the specified item,
   * else false.
   */
  function deleteItemAllowed($user,$item) {
  	$rank = $this->getRankByID($user["rankID"]);
  	
	if(($rank["delete_all_items"] == 1) ||
	   ($rank["delete_own_item"] == 1 && $item["userID"] == $user["userID"]))
	    return true;
	    
	return false;
  }
  
  /**
   * Returns true, if the user is allowed to delete the comments for a
   * specified item, else false.
   * 
   * @param user An array containing all user data.
   * @param $item An array containing the item data.
   * @return true, if the user is allowed to delete the comments for a
   * specified item, else false.
   */
  function deleteCommentsAllowed($user,$item) {
  	$rank = $this->getRankByID($user["rankID"]);
  	
	if(($rank["delete_all_comments"] == 1))
	    return true;
	    
	return false;
  }
  
  /**
   * Returns true, if the user is allowed to delete the comment.
   * 
   * @param user An array containing all user data.
   * @param $item An array containing the item data.
   * @return true, if the user is allowed to delete the comment, else false.
   */
  function deleteCommentAllowed($user, $comment) {
  	$rank = $this->getRankByID($user["rankID"]);
  	if($comment==false) return false;
	if(($rank["delete_all_comments"] == 1) ||
	   ($rank["delete_own_comments"] == 1 && $comment["userID"] == $user["userID"]))
	    return true;
	    
	return false;
  }
  
  /**
   * Returns true, if the user is allowed to create and add items.
   * 
   * @param user An array containing all user data.
   * @return true, if the user is allowed to create and add items. Else false.
   */
  function addItemAllowed($user) {
  	$rank = $this->getRankByID($user["rankID"]);
  	
	if($rank["add_item"] == 1)
	    return true;
	    
	return false;
  }
  
  /**
   * Returns true, if the user is allowed to edit this item.
   * 
   * @param user An array containing all user data.
   * @param $item An array containing the item data.
   * @return true, if the user is allowed to delete the comment, else false.
   */
  function editItemAllowed($user, $item) {
  	$rank = $this->getRankByID($user["rankID"]);
  	if(!$item) return false;
	if(($rank["edit_all_items"] == 1) ||
	   ($rank["edit_own_items"] == 1 && $item["userID"] == $user["userID"]))
	    return true;
	    
	return false;
  }
  
  /**
   * Returns true if the user is allowed to create new categories
   * 
   * @param user An array containing all user data.
   * @return true, if the user is allowed to create categories. Else false.
   */
   function createCategoryAllowed($user){
   	$rank = $this->getRankByID($user["rankID"]);
   	
   	if($rank["add_category"] == 1)
   		return true;
   	
   	return false;
   }
   
   /**
   * Returns true if the user is allowed to edit categories
   * 
   * @param user An array containing all user data.
   * @return true, if the user is allowed to edit categories. Else false.
   */
   function editCategoryAllowed($user){
   	$rank = $this->getRankByID($user["rankID"]);
   	
   	if($rank["edit_category"] == 1)
   		return true;
   	
   	return false;
   }
   
   /**
   * Returns true if the user is allowed to delete categories
   * 
   * @param user An array containing all user data.
   * @return true, if the user is allowed to delete categories. Else false.
   */
   function deleteCategoryAllowed($user){
   	$rank = $this->getRankByID($user["rankID"]);
   	
   	if($rank["delete_category"] == 1)
   		return true;
   	
   	return false;
   }
  
  /**
   * Checks if a rank already exists.
   * 
   * @param rank_name The rank to be tested.
   * @return True if it exists, else false.
   */
  function existsRank($rank_name) {
  	$sql = "Select * from ".$this->config["db_prefix"]."rank where rank_name = '".$rank_name."'";
  	$result = $this->dbCon->selectFromDB($sql);
  	while($row = mysql_fetch_array($result)){
  		return true;
  	}
  	return false;
  }
  
  /**
   * Tests wether the user is an admin or not.
   */
  function isAdmin($user) {
  	$rank = $this->getRankByID($user["rankID"]);
   	
   	if($rank["is_admin"] == 1)
   		return true;
   	
   	return false;
  }
  
  function addRank($rank_arr) {
  	$sql = "INSERT INTO ".$this->config["db_prefix"]."rank(`rank_name` , `add_item` , `edit_own_items` , `edit_all_items` , `delete_own_items` , `delete_all_items` , `add_category` , `edit_category` , `delete_category` , `delete_all_comments` , `delete_own_comments` , `add_comment`, `rank_visible`) " .
  			"VALUES('".$rank_arr["rank_name"]."'," .
  					"".$rank_arr["add_item"]."," .
  					"".$rank_arr["edit_own_items"]."," .
  					"".$rank_arr["edit_all_items"]."," .
  					"".$rank_arr["delete_own_items"]."," .
  					"".$rank_arr["delete_all_items"]."," .
  					"".$rank_arr["add_category"]."," .
  					"".$rank_arr["edit_category"]."," .
  					"".$rank_arr["delete_category"]."," .
  					"".$rank_arr["delete_all_comments"]."," .
  					"".$rank_arr["delete_own_comments"]."," .
  					"".$rank_arr["add_comment"].",1,0)";
  	$result = $this->dbCon->selectFromDB($sql);
  	return $result;
  }
  
  function editRank($rank_arr){
	$sql = "UPDATE ".$this->config["db_prefix"]."rank set " .
			"rank_name='".$rank_arr[rank_name]."', ".
			"add_item=".$rank_arr["add_item"]."," .
			"edit_own_items=".$rank_arr["edit_own_items"]."," .
			"edit_all_items=".$rank_arr["edit_all_items"]."," .
			"delete_own_items=".$rank_arr["delete_own_items"]."," .
			"delete_all_items=".$rank_arr["delete_all_items"]."," .
			"add_category=".$rank_arr["add_category"]."," .
			"edit_category=".$rank_arr["edit_category"]."," .
			"delete_category=".$rank_arr["delete_category"]."," .
			"delete_all_comments=".$rank_arr["delete_all_comments"]."," .
			"delete_own_comments=".$rank_arr["delete_own_comments"]."," .
			"add_comment=".$rank_arr["add_comment"].",".
			"rank_visible=1, ".
			"is_admin=0 ".
			"where rankID=".$rank_arr["rankID"];
	$result = $this->dbCon->selectFromDB($sql);
  	return $result;
  }
}
?>