<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/

/**
 * Created on 17.03.2005
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 0.1
 */

define("USERNAME_SESSION_NAME","irep_user");
define("PASSWORD_SESSION_NAME","irep_user_pass");
define("USERID_SESSION_NAME","irep_user_id");
class SessionManager {
    
    function SessionManager() {
    	session_start();
    }
    
    function setSessionVar($var_name,$var_value) {
    	$_SESSION[$var_name] = $var_value;
    }
    
    /**
     * Returns the session var with the name
     * $var_name or null, if it doesn't exist
     * 
     * @param $var_name the name of the variable
     * @return the variable or null, if it doesn't exist
     */
    function getSessionVar($var_name) {
    	if(isset($_SESSION[$var_name])) return $_SESSION[$var_name];
    	return false;
    }
    
    /**
     * Returns the username if an user is logged in.
     * Else it returns false.
     * 
     * @return the username or false
     */
    function getUsername() {
    	return $this->getSessionVar(USERNAME_SESSION_NAME);
    }
    
    /**
     * Returns the password of the user if he is logged in.
     * Else it returns false.
     * 
     * @return the password of the user or false
     */
    function getUserpass() {
    	return $this->getSessionVar(PASSWORD_SESSION_NAME);
    }
    
    /**
     * Returns the ID of the user if he is logged in.
     * Else it returns false.
     * 
     * @return the ID of the user or false
     */
    function getUserID() {
    	return $this->getSessionVar(USERID_SESSION_NAME);
    }
    
    /**
     * Adds the users data to the session.
     * 
     * @param username the username
     * @param password the password
     * @param user_id the user id
     */
    function addUser($username,$password,$user_id) {
    	$this->setSessionVar(USERNAME_SESSION_NAME,$username);
    	$this->setSessionVar(USERID_SESSION_NAME,$user_id);
    	$this->setSessionVar(PASSWORD_SESSION_NAME,$password);
    }
    
    /**
     * Returns true if an user is logged in, else false.
     */
    function loggedIn() {
    	if(!$this->getUsername() && !$this->getUserpass() && !$this->getUserID()) return false;
    	return true;
    }
    
    /**
     * Stops the session and deletes all variables.
     */
    function stopSession() {
    	session_destroy();
    }
}
?>