<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/

/**
 * Created on 19.04.2005
 * 
 * This class is the one that parses the Special Syntax 
 * 
 * To use this class, it is required, that a connection
 * to the I-Reporter Database has already been established.
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 0.1
 */
 
class SpecialSyntaxParser {
	
	var $dbCon;
	var $config;
	var $cat_manager;

    function SpecialSyntaxParser($dbCon, $config) {
    	$this->dbCon = $dbCon;
    	$this->config = $config;
    	
    	require_once($config["proj_path"].'/classes/CategoryManager.class.php');
    	$this->cat_manager = new CategoryManager($this->dbCon, $this->config);
    	require_once($config["proj_path"].'/classes/ItemManager.class.php');
    	$this->item_manager = new ItemManager($this->dbCon, $this->config, $this->dbCon->getOptionsArray());
    }
    
    
   /**
     * This function parses the text of the item,
     * and checks if the I-Reporter special syntax is used.
     * If yes, it recognizes the specified data and sets 
     * the appropriate fields in the array $temp_data
     * 
     * @param the string that should be parsed
     * @return Array containing all the values that were declared with special syntax
     */
    function parseStaticSpecialSyntax($string){
    	$temp_data = array();
    	if (preg_match("/\[([ctlCTL].+)\]/", $string)){
			preg_match_all("/\[([ctlCTL]):([^\[\]]+)\]/", $string, $result, PREG_SET_ORDER);
			$temp_data["item_text"] = preg_replace("/\[[ctlCTL]:([^\[\]]+\])+/", "", $string);
			for($i = 0; $i < count($result); $i++){
				if (strtolower($result[$i][1]) == "t") $temp_data["title"] = $result[$i][2];
				else if (strtolower($result[$i][1]) == "c"){
					if (!$this->cat_manager->isExistingCategory($result[$i][2])){
						$temp_data["category"] = $this->cat_manager->getDefaultCatID(); 
					}
					else {
						$temp_data["category"] = $this->cat_manager->isExistingCategory($result[$i][2]);
					}
				}
				else if (strtolower($result[$i][1]) == "l"){
					$temp_data["location"] = $result[$i][2];
				}
			}
		}
		return $temp_data;
    }
    
    /**
     * This function parses the optional existing [i:image.jpg] or [a:http://www.ireporter.de] special syntax 
     * construct and replaces it with the appropriate html <img> or <a> tag. 
     * 
     * @param $item_text The text of an item
     * @return $item_text The item text containing <img> or <a> tags
     */
    function parseRuntimeSpecialSyntax($string, $itemID, $show_images=true){
    	$sql = "Select * from ".$this->config["db_prefix"]."mediadata where itemID = ".$itemID;
    	$dbresult = $this->dbCon->SelectFromDB($sql);
    	$k = 0;
    	$filearray = array();
    	while($arr = mysql_fetch_array($dbresult)){
    		$filearray[$k] = $arr;
    		$k++;
    	}
    	preg_match_all("/\[([iIaA]):(http:\/\/[^\[\]]+|[^\[\]]+)\]/", $string, $result, PREG_SET_ORDER);
    	for($i = 0; $i < count($result); $i++){
     		if (strtolower($result[$i][1]) == "i"){   //image special syntax
     			$filename = $result[$i][2];
     			$alt_text = substr(strrchr($filename, ":"), 1);
				if ($alt_text == "") $alt_text = "Mobile Reporter Image"; //TODO aus config oder sprachdatei standard alt text auslesen
     			$pos = strrpos($filename, ":");
				if(!$pos){
					$filename = strtolower($filename);
				}
				else {
					$filename = substr($filename, 0, $pos);
				}
				if(count($filearray) > 0){
					for($m = 0; $m < count($filearray); $m++){
	 					if($filename == strtolower(substr($filearray[$m]["url"], strrpos($filearray[$m]["url"], "/")+1))){
	 						if($show_images)$string = preg_replace("/\[([iI]):([^\[\]]+)\]/", "<img src='".$filearray[$m]["url"]."' alt='$alt_text'/>", $string,1);
	 					    else $string = preg_replace("/\[([iI]):([^\[\]]+)\]/", "", $string,1);
	 					} 
	 					else $string = preg_replace("/\[([iI]):([^\[\]]+)\]/", "", $string,1);
	 				}
				} else $string = preg_replace("/\[([iI]):([^\[\]]+)\]/", "", $string,1);     			
     		}
     		else if (strtolower($result[$i][1]) == "a"){   //link special syntax 
     			$url = $result[$i][2];
     			
     			$media = $this->item_manager->getMediaByItem($itemID);
     			if(count($media) > 0){
     				//print_r($media);
     				for($i = 0; $i < count($media); $i++){
     					if($url == $media[$i]["filename"]){
     						$url = substr($media[$i]["url"], 2);
     						$description = $result[$i][2];
     					}	
     				}
     			}
     			//echo "url = $url";
     			
				$www = false;
				$http = false;
				if(preg_match("/http:\/\//i", $url)){
					$url = preg_replace("/http:\/\//i", "", $url);
					$http = true;
				}
				if(preg_match("/www./i", $url)){
					$url = preg_replace("/www./i", "", $url);
					$www = true;
				}
				if(!isset($description))$description = substr(strrchr($url, ":"), 1);
				if ($description == "") $description = $result[$i][2];
				$pos = strrpos($url, ":");
				if(!$pos){
					if($http && $www) $url = "http://www.".$url;
					else if($http && !$www) $url = "http://".$url;
					else if(!$http && $www) $url = "http://www.".$url; 
				}
				else {
					if($http && $www) $url = "http://www.".substr($url, 0, $pos);
					else if($http && !$www) $url = "http://".substr($url, 0, $pos);
					else if(!$http && $www) $url = "http://www.".substr($url, 0, $pos);
					else $url = substr($url, 0, $pos);
				}
				$string = preg_replace("/\[([aA]):(http:\/\/[^\[\]]+|[^\[\]]+)\]/", "<a href='$url'>$description</a>", $string, 1);
     		}
     	}
     	return $string;
    }
}
?>