<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/


/**
 * 
 * Created on 26.07.2005
 * 
 * This class provides thumbnail functionality. If no gdlib is installed on the server,
 * most of the functions will return false.
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 1.0
 */
 
class ThumbCreator {
	
	var $dbCon;
	var $config;
	var $options;
	
    function ThumbCreator($dbCon, $config, $options) {
    	$this->dbCon = $dbCon;
    	$this->config = $config;
    	$this->options = $options;
    }
    
    /**
     * Test, if gdlib is installed on the server.
     * @return true if it is installed, else false
     */
    function hasGDLib() {
    	if(function_exists("imagecreate")) return true;
    	return false;
    }
    
    /**
     * If the server supports gdlib, this method creates a thumbnail out of an image file
     * and saves it to the right directory.
     * 
     * @param media_path the path to the directory of the media.
     * for example: /media/cat1/item46
     * @param from_file the filename of the image to create a thumbnail for
     * for example: test.gif
     */
    function createThumbnail($media_path, $from_file) {
	    if(!$this->hasGDLib()) return false;
	    $thumb_path = $media_path ."thumb/";
	    if(!file_exists($thumb_path)) {
    		mkdir($thumb_path);
    	}
	    $imginfo = getimagesize($media_path.$from_file);
	    if(!$imginfo) return false;
//	    print_r($imginfo);
//	    print_r($imginfo);
	    switch ($imginfo[2]) {
	    	case 1:     // gif
	    	    $img_in = imagecreatefromgif($media_path.$from_file);
		        break;
		    case 2:     // jpg
		        $img_in = imagecreatefromjpeg($media_path.$from_file);
		        break;
		    case 3:     // png
		        $img_in = imagecreatefrompng($media_path.$from_file);
		        break;
		    default:
		        return false;
		}
	    
	    // resizing necessary?
	    if($this->options["thumb_w"] >= imagesx($img_in) && $this->options["thumb_h"] >= imagesy($img_in))
	      return false;
	    
	    $sizes = $this->computeSides($imginfo);
	    
	    if(function_exists("imagecreatetruecolor"))
	      $img_out = imagecreatetruecolor($sizes["w"], $sizes["h"]);
	    else
	      $img_out = imagecreate($sizes["w"], $sizes["h"]);
	    
	    if(function_exists("imagecopyresampled"))
	      imagecopyresampled($img_out,$img_in,0,0,0,0,imagesx($img_out),
	                     imagesy($img_out),imagesx($img_in),imagesy($img_in));
	    else
	      imagecopyresized($img_out,$img_in,0,0,0,0,imagesx($img_out),
	                     imagesy($img_out),imagesx($img_in),imagesy($img_in));
	                     
	    imagejpeg($img_out,$thumb_path."tn_".$from_file);
    }
    
    /**
     * Computes the width and height of the sample.
     * 
     * @param $imginfo The image infop created by getimagesize().
     * @return returns an array sizes with ["w"] and ["h"]
     */
    function computeSides($imginfo) {
    	$width = $imginfo[0];
    	$height = $imginfo[1];
    	if($width > $this->options["thumb_w"]) {
    		$perc =  $this->options["thumb_w"] / $width;
    		$height = $perc * $height;
    		$width = $this->options["thumb_w"];
    	}
    	
    	if($height > $this->options["thumb_h"]) {
    		$perc = $this->options["thumb_h"] / $height;
    		$width = $perc * $width;
    		$height = $this->options["thumb_h"];
    	}
//    	echo $height." : ".$width;
        return array("w" => $width, "h" => $height);
    }
    
    /**
     * Tests if the a file is an image file.
     * 
     * @param file The file to check.
     * @return true if the file is an image, else false
     */
    function isImgFile($file) {
    	if(function_exists("getimagesize")) {
		   $imginfo = getimagesize($file);
		   if($imginfo != false && $imginfo[2] >= 1 && $imginfo[2] <= 3){
		   	   return true;
		   }
		} else {
	    	$img_formats = array(0 => ".gif", 1 => ".jpeg", 2 => ".jpg", 3 => ".png");
			$low_string = strtolower($media["filename"]);
			foreach($img_formats as $format) {
		  		$pos = strripos($low_string,$format);
		  		if($pos != false) {
		  		  if(strlen($low_string) == $pos+strlen($format)) {
		  		  	 return true;
		  		  }
		  		}
		  	}
		}
		
		return false;
    }
    
    /**
     * Depending on the server this method creates a <img ../> tag,
     * that contains either the thumbnail image or width and height attributes,
     * that fake a thumbnail image.
     * 
     * @param item An array presenting the item.
     * @param item_manager An instance of the ItemManager class.
     * @return The image tag that represents the thumb or false if there is no image
     * belonging to this item, or an error occured.
     */
    function createThumbTag($item,$item_manager) {
    	$path = "./media/cat".$item["catID"]."/item".$item["itemID"]."/";
    	$thumb_path = $path."thumb/";
    	
    	$main_media = false;
    	// first search for an image
    	if(isset($item["main_img"]) && $item["main_img"] != null) {
    		$main_media = $item_manager->getMediaByID($item["main_img"]);
    	}
    	
    	if($main_media == false) {
			$medias = $item_manager->getMediaByItem($item["itemID"]);
	    	if($medias!=null) {
		    	foreach($medias as $media) {
					if($this->isImgFile($media["url"])) {
						$main_media = $media;
						break;
					}
				}
	    	}
		}
		
		if($main_media == false) return false;
    	
    	// if an image has been found, create the img tag
    	
    	$thumb_file = $thumb_path."tn_".$main_media["filename"];
		$orig_file = $path.$main_media["filename"];
		if(file_exists($thumb_file)) { // 1. does a thumbnail exist?
			if(function_exists("getimagesize")) {
				$imginfo = getimagesize($thumb_file);
				if($imginfo != false && $imginfo[2] >= 1 && $imginfo[2] <= 3)
				   return "<img src=\"".$thumb_file."\" ".$imginfo[3]." />";
			} else {
				   return "<img src=\"$thumb_file\" />";
			}
		} else if(file_exists($orig_file)) { // 1. perhaps the original file exists
		    if(function_exists("getimagesize")) {
		    	$imginfo = getimagesize($orig_file);
		    	if($imginfo != false && $imginfo[2] >= 1 && $imginfo[2] <= 3) {
			    	$sides = $this->computeSides($imginfo);
			    	return "<img src=\"".$orig_file."\" height=\"".$sides["h"]."\" width=\"".$sides["w"]."\" />";
		    	}
		    } else
		        return "<img src=\"".$orig_file."\" />";
		}
		
    	return false;
    }
}
?>