<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/

/**
 * Created on 17.08.2005
 *
 * This script shows a specific category or an overview over all categories.
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 0.1
 */
 
 
 require_once('./classes/DisplayView.class.php');
 require_once('./classes/SpecialSyntaxParser.class.php');
 require_once('./classes/CommentManager.class.php');
 require_once('./classes/ThumbCreator.class.php');
 
 
 class IndexView extends DisplayView {
	
	var $item_view_user_factor = 0.6;
	var $item_view_visitor_factor = 0.4;
	var $comment_count_factor = 1.1;
	
	var $thumb_creator;
	var $comment_manager;
	var $special_syntax_parser;
	var $truncate_size = 800;
 	
 	function IndexView() {
 		parent::DisplayView();
 		$this->setTitle($this->lang_main["index_title"]);
 		global $config;
 		$this->special_syntax_parser = new SpecialSyntaxParser($this->dbCon,$config);
 		$this->thumb_creator = new ThumbCreator($this->dbCon,$config,$this->options);
 		$this->comment_manager = new CommentManager($this->dbCon,$config,$this->options);
 	}
 	
 	function display() { 	
 		global $config;
 		if($this->dbCon->itsTimeToCheck() && $this->options["email_check_on"] == 1) {
			require_once($config["proj_path"].'/classes/CheckMail.class.php');
			$checkmail = new CheckMail($this->dbCon, $config);
			$checkmail->checkForMails();
 		}
 		//if ($this->dbCon->itsTimeToCheck()) $this->item_manager->checkMails();
 		$this->getItems();
 		
 		$this->my_smarty->display($this->options["template"].'/index.tpl');
 	}
 	
 	function getItems(){
 		if($this->options["ranking_on"] == 0){
 			$items = $this->item_manager->getAllItems(0,5);
 			if($items != null){
 				$items = $this->item_manager->createDisplayItems($items, $this->special_syntax_parser,$this->comment_manager,$this->thumb_creator, $this->lang_main);
 				$this->my_smarty->assign("last_items_title", $this->lang_main["last_items_title"]);
		 		$this->my_smarty->assign("items", $items);
		 		$this->my_smarty->assign("truncate_size",$this->truncate_size);
		 		if($this->options["comments_allowed"] == 1){ // are comments allowed?
			    	$this->my_smarty->assign('comments_allowed',"true");
			    	$this->my_smarty->assign('comments_label',$this->lang_main["comments_label"]);
		 		}
 			}
 			else $this->my_smarty->assign("no_item", $this->lang_main["no_item"]);
 		}
 		else {
	 		$act_item = $this->item_manager->getMostActualItem($this->special_syntax_parser,$this->comment_manager,$this->thumb_creator, $this->lang_main, $this->item_view_user_factor, $this->item_view_visitor_factor);
	 		if($act_item != null){
	 			$act_item = $this->item_manager->createSingleDisplayItem($act_item, $this->special_syntax_parser,$this->comment_manager,$this->thumb_creator, $this->lang_main);
		 		
		 		//get the most viewed Items to determine the top 3 of Items
		 		$items = $this->item_manager->getMostViewedItems($this->item_view_user_factor, $this->item_view_visitor_factor, $act_item["itemID"]);
		 		if($items != null){
		 			if($this->options["comments_allowed"] == 1){ // are comments allowed?
				    	$this->my_smarty->assign('comments_allowed',"true");
				    	$this->my_smarty->assign('comments_label',$this->lang_main["comments_label"]);
				    	$items = $this->addItemCommentCount($items);
			 		}
			 		if($this->options["rating_allowed"] == 1){
			 			$items = $this->addRatingCount($items);
			 		}
			 		
			 		$items = $this->item_manager->createDisplayItems($items, $this->special_syntax_parser,$this->comment_manager,$this->thumb_creator, $this->lang_main);
			 		$top3_items = $this->getTop3Items($items);
			 			 		
			 		if($top3_items != null){
			 			$this->my_smarty->assign('top3_items',$top3_items);
			 		}
			 		else {
			 			$this->my_smarty->assign("top3_noitem", $this->lang_main["top3_noitem"]);
			 		} 	
			 	}
		 		else {
		 			$this->my_smarty->assign("top3_noitem", $this->lang_main["top3_noitem"]);
		 		}
		 		
		 		$this->my_smarty->assign("truncate_size",$this->truncate_size);
		 		$this->my_smarty->assign("top3_title", $this->lang_main["top3_title"]);
		 		$this->my_smarty->assign("actual_item_title", $this->lang_main["act_item_title"]);
		 		$this->my_smarty->assign("act_item", $act_item);
	 		}
	 		else $this->my_smarty->assign("no_item", $this->lang_main["no_item"]);
 		} 		
 	}
 	
 	function comp($a,$b) {
		if($a["total_factor"] == $b["total_factor"]) return 0;
		return ($a["total_factor"] > $b["total_factor"]) ? -1 : 1;
	}
 	
 	function getTop3Items($items){
 		if($items != null){
 			usort($items,array("IndexView","comp"));
 			$top3_items = array();
 			for($i=0;$i<count($items);$i++){
	 			if($i <= 2){
	 				$top3_items[$i] = $items[$i];
	 			}
 			}
	 		if(count($top3_items) > 0) return $top3_items;
	 		return null;
 		}
 		else return null;
  	}
  	
  	function addItemCommentCount($items){
  		$new_items = array();
  		foreach($items as $item){
  			$total_factor = $item["total_factor"];
  			$comment_count = $this->comment_manager->getCommentCount($item["itemID"]);
  			if($comment_count > 0) $item["total_factor"] = $total_factor * ($comment_count * $this->comment_count_factor);
  			$new_items[count($new_items)] = $item;
  		}
  		if(count($new_items) > 0) return $new_items;
  		return false;
  	}
  	
  	function addRatingCount($items){
  		$new_items = array();
  		foreach($items as $item){
  			$total_factor = $item["total_factor"];
  			if($rating = $this->item_manager->getRatingByItem($item["itemID"])){
  				if($rating[1] == 1 && $rating[0] == 0) $rating_factor = 1.5;
  				else if($rating[0] == 1 && $rating[1] == 0) $rating_factor = 0.5;
  				else if($rating[1] > 0 && $rating[0] == 0) $rating_factor = $rating[1]/1;
  				else if($rating[0] > 0 && $rating[1] == 0) $rating_factor = 1/$rating[0];
  				else if($rating[1] > $rating[0]) $rating_factor = $rating[1]/$rating[0];
  				else if($rating[0] > $rating[1]) $rating_factor = $rating[1]/$rating[0];
  				$item["total_factor"] = $total_factor * $rating_factor;
  				$item["rating_factor"] = $rating_factor;
  			}
  			$new_items[count($new_items)] = $item;
  		}
  		if(count($new_items) > 0) return $new_items;
  	}
 	
 	function showConfirmDialogue() {
		if(!$this->checkDeleteRights()) return;
		$this->my_smarty->assign('yes_button',$this->lang_main["yes_button"]);
		$this->my_smarty->assign('no_button',$this->lang_main["no_button"]);
//	    if(isset($_REQUEST["catID"]) && $_REQUEST["catID"] != "")
//	    	$this->my_smarty->assign('dialogue_url','cat.php?catID='.$_REQUEST["catID"]);
//	    else 
	    $this->my_smarty->assign('dialogue_url','cat.php');
	    $this->my_smarty->assign('dialogue_do_value',"delete_confirmed");
		$this->my_smarty->assign('confirm_id',$_REQUEST["deleteID"]);
 		$this->my_smarty->assign('dialogue_title',$this->lang_main["dialogue_delete_title"]);
		$this->my_smarty->assign('dialogue_text',$this->lang_main["dialogue_delete_category"]);
 	}
 	
 	
 	
 	function getScriptName() {
   	   return "index.php";
    }
}

 $index_view = new IndexView();
 $index_view->display();
?>