<?php
/********************************************************************************
* Copyright (C) 2005 Alexander De Luca, Michael Mller

* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*********************************************************************************/

/**
 * Created on 04.05.2005
 *
 * This class shows one whole item including the comments.
 * 
 * @author Alexander De Luca, Michael Mueller
 * @version 0.8
 */
 
 require_once('./classes/ViewManager.class.php');
 require_once('./classes/SpecialSyntaxParser.class.php');
 require_once('./classes/CommentManager.class.php');
 require_once('./classes/DisplayView.class.php');
 require_once('./classes/FeedCreator.class.php');
 
 class ItemView extends DisplayView {
 	
 	var $special_syntax_parser;
 	var $view_manager; // counts views and visits
 	var $comment_manager;
 	var $item_data;
 	var $show_item = true;
 	
 	function ItemView() {
 		global $config;
 		parent::DisplayView();
 		$this->special_syntax_parser = new SpecialSyntaxParser($this->dbCon,$config);
 		$this->view_manager = new ViewManager($this->dbCon,$config);
 		$this->comment_manager = new CommentManager($this->dbCon,$config,$this->options);
 		$username_act = $this->session_manager->getUsername();
 		$password_act = $this->session_manager->getUserpass();
 		$userID_act = $this->session_manager->getUserID();
 	}
 	
 	function display() {
 		$this->item_data = $this->getItem();
 		if(!$this->item_data) return;
 		
 		// add a view to this item
 		$this->view_manager->addView($_REQUEST["itemID"],$this->valid_user);
 		
 		 		
 		if(isset($_REQUEST["do"]))
 			$this->handleDoCommands();
 		if(isset($_REQUEST["rate"]))
 			$this->rateItem();
 		
 		if(!$this->show_item) return;
 		$this->showItem($this->item_data);
 		
 		$this->showComments();
 		$this->showRating();
 		
 		$this->my_smarty->display($this->options["template"].'/item.tpl');
 	}
 	
 	/**
 	 * It is possible, that different do values are submitted in a POST request.
 	 * This method decides what to do when finding do values.
 	 */
 	function handleDoCommands() {
 		if(isset($_POST["do"]) && $_POST["do"] == "addcomment" && $this->options["comments_allowed"] == 1) $this->addComment();
 		else if($this->valid_user && isset($_GET["do"]) && ($_GET["do"] == "deletebox" || $_GET["do"] == "delcomsbox")) $this->showConfirmDialogue();
 		else if($this->valid_user && isset($_POST["do"]) && $_POST["do"] == "delcombox") $this->showConfirmDialogue();
 		else if($this->valid_user && isset($_POST["do"]) && $_POST["do"] == "delete" && isset($_POST["yes_button"])) $this->deleteItem();
 		else if($this->valid_user && isset($_POST["do"]) && $_POST["do"] == "deletecoms" && isset($_POST["yes_button"])) $this->deleteComments();
 		else if($this->valid_user && isset($_POST["do"]) && $_POST["do"] == "deletecom" && isset($_POST["yes_button"])) $this->deleteComment();
 	}
 	
 	function deleteItem() {
 		global $config;
 		if(!$this->rank_manager->deleteItemAllowed($this->user,$this->item_data))
		   return;
 		if(!$this->item_manager->deleteItem($_POST["confirm_id"])) {
 			$this->showError($this->lang_error["item_delete_error"]);
 			return;
 		}
		$this->createHeader($this->lang_main["success_title"]);
		
	 	$this->my_smarty->assign('success_title',$this->lang_main["success_title"]); 
	    $this->my_smarty->assign('success_message',$this->lang_main["dialogue_delete_item_success"]);
	    $this->my_smarty->display($this->options["template"].'/success.tpl');
	    $this->show_item = false;
	    
 	}
 	
 	function deleteComments() {
 		if(!$this->rank_manager->deleteCommentsAllowed($this->user,$this->item_data))
		   return;
 		if(!$this->comment_manager->deleteCommentsByItemID($_POST["confirm_id"])) {
 			$this->showError($this->lang_error["comments_delete_error"]);
 			
 			return;
 		}
 		$this->showSuccess($this->lang_main["dialogue_delete_comments_success"]);
 	}
 	
 	function deleteComment() {
 		if(!$this->rank_manager->deleteCommentAllowed($this->user,
                                                      $this->comment_manager->getComment($_POST["confirm_id"])))
		   return;
 		if(!$this->comment_manager->deleteComment($_POST["confirm_id"])) {
 			$this->showError($this->lang_error["comment_delete_error"]);
 			
 			return;
 		}
 		$this->showSuccess($this->lang_main["dialogue_delete_comment_success"]);
 	}
 	
 	function showConfirmDialogue() {
		$this->my_smarty->assign('yes_button',$this->lang_main["yes_button"]);
		$this->my_smarty->assign('no_button',$this->lang_main["no_button"]);
		$url_add = "";
		if(isset($_REQUEST["showcomments"]) && $_REQUEST["showcomments"] == true) $url_add = "&amp;showcomments=true";
	    $this->my_smarty->assign('dialogue_url',$this->getScriptName()."?itemID=".$_REQUEST["itemID"].$url_add);
		$this->my_smarty->assign('confirm_id',$_REQUEST["itemID"]);
 		if(isset($_GET["do"]) && $_GET["do"] == "deletebox") {
 			if(!$this->rank_manager->deleteItemAllowed($this->user,$this->item_data))
 			   return;
			$this->my_smarty->assign('dialogue_title',$this->lang_main["dialogue_delete_title"]);
			$this->my_smarty->assign('dialogue_text',$this->lang_main["dialogue_delete_item"]);
			$this->my_smarty->assign('dialogue_do_value',"delete");
 		} else if(isset($_GET["do"]) && $_GET["do"] == "delcomsbox") {
 			if(!$this->rank_manager->deleteCommentsAllowed($this->user,$this->item_data))
 			   return;
			$this->my_smarty->assign('dialogue_title',$this->lang_main["dialogue_delete_title"]);
			$this->my_smarty->assign('dialogue_text',$this->lang_main["dialogue_delete_comments"]);
			$this->my_smarty->assign('dialogue_do_value',"deletecoms");
 		}  else if(isset($_POST["do"]) && $_POST["do"] == "delcombox") {
 			if(!isset($_POST["commentID"]) || (!$this->rank_manager->deleteCommentAllowed($this->user,
 			$this->comment_manager->getComment($_POST["commentID"]))))
 			   return;
			$this->my_smarty->assign('dialogue_title',$this->lang_main["dialogue_delete_title"]);
			$this->my_smarty->assign('dialogue_text',$this->lang_main["dialogue_delete_comment"]);
			$this->my_smarty->assign('confirm_id',$_POST["commentID"]);
			$this->my_smarty->assign('dialogue_do_value',"deletecom");
 		}
 	}
 	
 	function showItem($item_data) {
 		global $php_extension;
 		$this->setTitle($item_data["title"]);
		
		$this->my_smarty->assign('item_title',$item_data["title"]); 
		$this->my_smarty->assign('item_author',$item_data["username"]);
		if($this->user_manager->hasEmail($item_data["userID"]) && $this->valid_user){ //TODO && $this->options["sendmail_allowed"] == 1){
			$this->my_smarty->assign('authorID', $item_data["userID"]);
			$this->my_smarty->assign('back_path', base64_encode($this->createSelfURL()));
		}
		$this->my_smarty->assign('item_date',$item_data["item_date"]);
		$parsed_text = $this->special_syntax_parser->parseRuntimeSpecialSyntax($item_data["item_text"],$_REQUEST["itemID"]);
		
		$this->my_smarty->assign('item_text',$parsed_text);
		$this->my_smarty->assign('comments_label',$this->lang_main["comments_label"]);
		$this->my_smarty->assign('item_id',$_REQUEST["itemID"]);
		
		if($this->valid_user) $this->my_smarty->assign('stat_url',"statistic.".$php_extension."?itemID=".$item_data["itemID"]); 
		
		$this->my_smarty->assign('item_media',$this->item_manager->getMediaByItem($_REQUEST["itemID"]));
		if($this->options["comments_allowed"] == 1)  // are comments allowed?
		    	$this->my_smarty->assign('add_comments',"true");
 	}
 	
 	function showComments() {
 		$this->my_smarty->assign('comments_count',$this->comment_manager->getCommentCount($_REQUEST["itemID"]));
 
		 if(isset($_REQUEST["showcomments"]) && $_REQUEST["showcomments"] == "true") {
		    $this->my_smarty->assign('show_comments',"true");
		    if($this->options["comments_allowed"] == 1) { // are comments allowed?
		    	if(!$this->valid_user) $this->my_smarty->assign('show_user_input',"true");
			    $this->my_smarty->assign('username_label', $this->lang_main["username_label"]);
			    $this->my_smarty->assign('email_label',$this->lang_main["email_label"]);
			    $this->my_smarty->assign('comment_label',$this->lang_main["comment_label"]);
			    $this->my_smarty->assign('submit_button',$this->lang_main["submit_button"]);
		    }
		    
		  
		    $display_comments = $this->comment_manager->getDisplayComments($_REQUEST["itemID"],$this->lang_main, $this->item_manager);
		    
		    foreach($display_comments as $key => $value) {
		    	if($this->valid_user && $this->rank_manager->deleteCommentAllowed($this->user,$value))
		    	  $display_comments[$key]["comment_del_button"] = "true";
		    }
		    if($display_comments != false) $this->my_smarty->assign('comments',$display_comments);
		    
		 }
 	}
 	
 	function showRating(){
 		if($this->valid_user && $this->options["rating_allowed"] == 1 && $this->item_manager->checkItemRating($this->session_manager->getUserID(), $_REQUEST["itemID"])){
 			$this->my_smarty->assign("rating", "true");
 			$this->my_smarty->assign("itemID", $_REQUEST["itemID"]);
 			$this->my_smarty->assign("rate_this_item", $this->lang_main["rate_this_item"]);
 			$this->my_smarty->assign("rating_good_item", $this->lang_main["rating_good_item"]);
 			$this->my_smarty->assign("rating_bad_item", $this->lang_main["rating_bad_item"]);
 		}
 	}
 	
 	function rateItem(){
 		global $config;
 		if($_REQUEST["rate"] != ""){
 			$userID = $this->session_manager->getUserID();
	 		if($this->valid_user && $this->options["rating_allowed"] == 1 && $this->item_manager->checkItemRating($userID, $_REQUEST["itemID"]) && isset($_REQUEST["rate"])){
	 			$sql = "insert into ".$config["db_prefix"]."rating (itemID, userID, rating) values (".$_REQUEST["itemID"].", ".$userID.", ".$_REQUEST["rate"].")";
	 			$ok = $this->dbCon->insertIntoDB($sql);
	 			if($ok){
	 				$this->my_smarty->assign("success_title", $this->lang_main["success_title"]);
	 				$this->my_smarty->assign("success_message", $this->lang_main["item_rating_success"]);
	 			}	
	 			else {
	 				$this->my_smarty->assign("error_message", $this->lang_error["item_rating_error"]);
	 				$this->my_smarty->assign("success_message", $this->lang_error["error_title"]);
	 			}
	 		}
 		}
 	}
 	
 	function addComment() {
	 	if($this->checkInputfields()) {
	 		if($this->valid_user) $this->comment_manager->addComment($_POST["itemID"],$_POST["comment"],"",$this->session_manager->getUserID());
		    else {
		    	$this->comment_manager->addComment($_POST["itemID"],$_POST["comment"],$_POST["username"], false, $_POST["email"]);
		    }
		    $this->showSuccess($this->lang_main["comment_added"]);
	 	}
 	}
 	
 	function getItem() {
 		$item_data = false;
 		if(!isset($_REQUEST["itemID"]) || !$item_data = $this->item_manager->getItemByID($_REQUEST["itemID"])) {
		 	$this->setTitle($this->lang_error["error_title"]);
		 	
		 	$this->my_smarty->assign('error_title',$this->lang_error["item_not_existing_title"]); 
		    $this->my_smarty->assign('error_message',$this->lang_error["item_not_existing"]);
		    $this->my_smarty->display($this->options["template"].'/error.tpl');
		 	return false;
		}
		if(preg_match("/l/", $this->options["date_format"])){
			$item_data["item_date"] = $this->item_manager->getDateString($this->options["date_format"], $item_data["item_timestamp"], $this->lang_main);
		}
		else $item_data["item_date"] = date($this->options["date_format"], $item_date["item_timestamp"]);
		return $item_data;
 	}
 	
 	function checkInputfields() {
	 	$missing_field = false;
		if(!isset($_POST["comment"]) || $_POST["comment"]=="") {
		    $this->my_smarty->assign("comment_class","red_border");
		    $missing_field = true;
		}
		
		if(!$this->valid_user &&(!isset($_POST["username"]) || $_POST["username"]=="")) {
		    $this->my_smarty->assign("username_class","red_border");
		    $missing_field = true;
		}
	 	
	 	if($missing_field) {
	 		$this->showError($this->lang_error["missing_input_fields"]);
	 		return false;
	 	}
	 	
	 	return true;
	}
	
	function getScriptName() {
		global $php_extension;
   	    return "item.".$php_extension;
    }
 }
 
 $item_view = new ItemView();
 $item_view->display();
 
?>